import random

import pytest

from indy_common.authorize import auth_map
from indy_common.authorize.auth_constraints import accepted_roles, ConstraintsEnum, \
    AuthConstraint
from indy_node.test.auth_rule.auth_framework.add_roles import AddNewTrusteeTest, AddNewStewardTest, \
    AddNewEndorserTest, AddNewIdentityOwnerTest, AddNewNetworkMonitorTest
from indy_node.test.auth_rule.auth_framework.basic import roles_to_string, AuthTest
from plenum.common.exceptions import RequestRejectedException

from indy_node.test.helper import build_auth_rule_request_json


class EditRoleTest(AuthTest):
    def __init__(self, action_id, env, add_new_role_cls):
        super().__init__(env, action_id)
        constraint = auth_map.auth_map[action_id]
        self.constraint = constraint
        self.role = self.action.old_value
        # In auth action IDENTITY_OWNER is a '', but in accepted_roles it's None
        self.role = self.role if self.role != '' else None
        self.role_string = roles_to_string[self.role]
        self.role_to_change = self.action.new_value
        self.role_to_change = self.role_to_change if self.role_to_change != '' else None
        self.role_to_change_string = roles_to_string[self.role_to_change]
        self.need_to_be_owner = constraint.need_to_be_owner if hasattr(constraint, 'need_to_be_owner') else False
        self.other_roles = []
        self.default_who_can_wallet = None
        self.new_who_can_wallet = None
        self.new_default_did = None
        self.add_new_role_cls = add_new_role_cls

    def prepare(self):
        default_constraint_roles = []
        self.get_default_roles(self.constraint, default_constraint_roles)
        default_role = random.choice(default_constraint_roles)
        default_role = default_role if default_role != '*' else self.role
        self.other_roles = set(accepted_roles).difference(set(default_constraint_roles).union({'*', default_role}))
        self.new_default_did = self.create_role()
        if self.need_to_be_owner:
            self.default_who_can_wallet = (self.trustee_wallet[0], self.new_default_did)
        else:
            self.default_who_can_wallet = self.env.role_to_wallet[default_role]
        self.default_auth_rule = self.get_default_auth_rule()
        self.changed_auth_rule = self.get_changed_auth_rule()

        self.default_edit_nym_1 = self._get_default_edit_nym()
        self.default_edit_nym_2 = self._get_default_edit_nym()
        self.nym_for_new_rule = self._get_nym_for_new_rule()

        self.return_to_default_nym_1 = self._return_to_default_nym()
        self.return_to_default_nym_2 = self._return_to_default_nym()
        self.return_to_nym_for_new_rule = self._return_to_default_nym()

    def run(self):

        # Step 1. Change AUTH_RULE
        self.send_and_check(self.changed_auth_rule, wallet=self.trustee_wallet)

        # Step 2. Check, that we cannot edit nym in default way
        with pytest.raises(RequestRejectedException):
            self.send_and_check(self.default_edit_nym_1, wallet=self.default_who_can_wallet)

        # Step 3. Check, that new auth rule worked
        self.send_and_check(self.nym_for_new_rule, wallet=self.new_who_can_wallet)

        # Step 4. Return to default role
        if self.role != self.role_to_change:
            self.send_and_check(self.return_to_nym_for_new_rule, wallet=self.trustee_wallet)

        # Step 5. Return back to default auth rules
        self.send_and_check(self.default_auth_rule, wallet=self.trustee_wallet)

        # Step 6. Check, that now we can editing
        self.send_and_check(self.default_edit_nym_2, wallet=self.default_who_can_wallet)

        # Step 7. Return to default value
        if self.role != self.role_to_change:
            self.send_and_check(self.return_to_default_nym_2, wallet=self.trustee_wallet)

    def result(self):
        pass

    def get_default_roles(self, constraint, d_roles):
        if constraint.constraint_id != ConstraintsEnum.ROLE_CONSTRAINT_ID:
            for a_c in constraint.auth_constraints:
                self.get_default_roles(a_c, d_roles)
        else:
            d_roles.append(constraint.role)
        return d_roles

    def create_role(self):
        add_new_role = self.add_new_role_cls(self.env)
        new_role_nym = add_new_role.get_nym()
        res = add_new_role.send_and_check(new_role_nym, wallet=self.trustee_wallet)
        new_did = res[1]['result']['txn']['data']['dest']
        return new_did

    def _get_nym_for_new_rule(self):
        return self._build_nym(self.new_who_can_wallet,
                               self.role_to_change_string,
                               self.new_default_did)

    def _get_default_edit_nym(self):
        return self._build_nym(self.default_who_can_wallet,
                               self.role_to_change_string,
                               self.new_default_did)

    def _return_to_default_nym(self):
        return self._build_nym(self.trustee_wallet,
                               self.role_string,
                               self.new_default_did)

    def get_changed_auth_rule(self):
        new_role = random.choice(list(self.other_roles))
        self.new_who_can_wallet = self.env.role_to_wallet[new_role]
        constraint = AuthConstraint(role=new_role,
                                    sig_count=1,
                                    need_to_be_owner=False)
        return build_auth_rule_request_json(
            self.looper, self.trustee_wallet[1],
            auth_action=self.action.prefix,
            auth_type=self.action.txn_type,
            field=self.action.field,
            old_value=self.action.old_value,
            new_value=self.action.new_value,
            constraint=constraint.as_dict
        )


class EditTrusteeToStewardTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewTrusteeTest)


class EditTrusteeToEndorserTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewTrusteeTest)


class EditTrusteeToNetworkMonitorTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewTrusteeTest)


class EditTrusteeToIdentityOwnerTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewTrusteeTest)


class EditStewardToTrusteeTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewStewardTest)


class EditStewardToNetworkMonitorTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewStewardTest)


class EditStewardToEndorserTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewStewardTest)


class EditStewardToIdentityOwnerTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewStewardTest)


class EditEndorserToTrusteeTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewEndorserTest)


class EditEndorserToStewardTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewEndorserTest)


class EditEndorserToNetworkMonitorTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewEndorserTest)


class EditEndorserToIdentityOwnerTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewEndorserTest)


class EditIdentityOwnerToTrusteeTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewIdentityOwnerTest)


class EditIdentityOwnerToStewardTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewIdentityOwnerTest)


class EditIdentityOwnerToEndorserTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewIdentityOwnerTest)


class EditIdentityOwnerToNetworkMonitorTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewIdentityOwnerTest)


class EditNetworkMonitorToTrusteeTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewNetworkMonitorTest)


class EditNetworkMonitorToStewardTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewNetworkMonitorTest)


class EditNetworkMonitorToEndorserTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewNetworkMonitorTest)


class EditNetworkMonitorToIdentityOwnerTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewNetworkMonitorTest)


class EditStewardToStewardTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewStewardTest)


class EditTrusteeToTrusteeTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewTrusteeTest)


class EditEndorserToEndorserTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewEndorserTest)


class EditNetworkMonitorToNetworkMonitorTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewNetworkMonitorTest)


class EditIdentityOwnerToIdentityOwnerTest(EditRoleTest):
    def __init__(self, env, action_id):
        super().__init__(action_id,
                         env,
                         AddNewIdentityOwnerTest)



import pytest
import json

from indy_node.test.validator_info.helper import sdk_get_validator_info
from plenum.common.exceptions import RequestRejectedException

from indy_common.constants import VALIDATOR_INFO
from plenum.common.constants import REPLY, TXN_TYPE, DATA
from plenum.common.types import f
from plenum.test.helper import sdk_gen_request, sdk_sign_and_submit_req_obj, \
    sdk_get_reply, sdk_send_signed_requests, sdk_get_and_check_replies


def test_validator_info_command(
        sdk_pool_handle, sdk_wallet_trustee, looper):
    req, resp = sdk_get_validator_info(looper,
                                 sdk_wallet_trustee,
                               sdk_pool_handle)
    _comparison_reply(resp, req)


def test_fail_validator_info_command(
        sdk_pool_handle, sdk_wallet_client, looper):
    with pytest.raises(RequestRejectedException) as excinfo:
        sdk_get_validator_info(looper,
                               sdk_wallet_client,
                               sdk_pool_handle)
    assert excinfo.match("Rule for this action is")


def _comparison_reply(responses, req_obj):
    for json_resp in responses.values():
        resp = json.loads(json_resp)
        assert resp["op"] == REPLY
        assert resp[f.RESULT.nm][f.IDENTIFIER.nm] == req_obj[f.IDENTIFIER.nm]
        assert resp[f.RESULT.nm][f.REQ_ID.nm] == req_obj[f.REQ_ID.nm]
        assert resp[f.RESULT.nm][TXN_TYPE] == VALIDATOR_INFO
        assert resp[f.RESULT.nm][DATA] is not None





import pytest
from indy_common.authorize.auth_actions import AuthActionAdd, AuthActionEdit
from plenum.common.constants import TRUSTEE, STEWARD, VERKEY

from indy_common.constants import ROLE, NYM, ENDORSER, NETWORK_MONITOR


@pytest.fixture(scope='module', params=[True, False])
def is_owner(request):
    return request.param


def test_make_trustee(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionAdd(txn_type=NYM,
                                                                 field=ROLE,
                                                                 value=TRUSTEE,
                                                                 is_owner=is_owner)])


def test_make_steward(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionAdd(txn_type=NYM,
                                                                 field=ROLE,
                                                                 value=STEWARD,
                                                                 is_owner=is_owner)])


def test_make_endorser(write_request_validation, req, is_owner):
    authorized = req.identifier in ("trustee_identifier", "steward_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionAdd(txn_type=NYM,
                                                                 field=ROLE,
                                                                 value=ENDORSER,
                                                                 is_owner=is_owner)])


def test_make_network_monitor(write_request_validation, req, is_owner):
    authorized = req.identifier in ("trustee_identifier", "steward_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionAdd(txn_type=NYM,
                                                                 field=ROLE,
                                                                 value=NETWORK_MONITOR,
                                                                 is_owner=is_owner)])


# Trustee tests
def test_change_trustee_to_trustee(write_request_validation, req, is_owner):
    authorized = is_owner
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=TRUSTEE,
                                                                  new_value=TRUSTEE,
                                                                  is_owner=is_owner)])


def test_change_trustee_to_steward(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=TRUSTEE,
                                                                  new_value=STEWARD,
                                                                  is_owner=is_owner)])


def test_change_trustee_to_endorser(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=TRUSTEE,
                                                                  new_value=ENDORSER,
                                                                  is_owner=is_owner)])


def test_change_trustee_to_network_monitor(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=TRUSTEE,
                                                                  new_value=NETWORK_MONITOR,
                                                                  is_owner=is_owner)])


def test_change_trustee_to_identity_owner(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=TRUSTEE,
                                                                  new_value='',
                                                                  is_owner=is_owner)])


# Steward tests
def test_change_steward_to_trustee(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=STEWARD,
                                                                  new_value=TRUSTEE,
                                                                  is_owner=is_owner)])


def test_change_steward_to_steward(write_request_validation, req, is_owner):
    authorized = is_owner
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=STEWARD,
                                                                  new_value=STEWARD,
                                                                  is_owner=is_owner)])


def test_change_steward_to_endorser(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=STEWARD,
                                                                  new_value=ENDORSER,
                                                                  is_owner=is_owner)])


def test_change_steward_to_network_monitor(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=STEWARD,
                                                                  new_value=NETWORK_MONITOR,
                                                                  is_owner=is_owner)])


def test_change_steward_to_identity_owner(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=STEWARD,
                                                                  new_value='',
                                                                  is_owner=is_owner)])


# Endorser tests
def test_change_endorser_to_trustee(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=ENDORSER,
                                                                  new_value=TRUSTEE,
                                                                  is_owner=is_owner)])


def test_change_endorser_to_steward(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=ENDORSER,
                                                                  new_value=STEWARD,
                                                                  is_owner=is_owner)])


def test_change_endorser_to_endorser(write_request_validation, req, is_owner):
    authorized = is_owner
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=ENDORSER,
                                                                  new_value=ENDORSER,
                                                                  is_owner=is_owner)])


def test_change_endorser_to_network_monitor(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=ENDORSER,
                                                                  new_value=NETWORK_MONITOR,
                                                                  is_owner=is_owner)])


def test_change_endorser_to_identity_owner(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=ENDORSER,
                                                                  new_value='',
                                                                  is_owner=is_owner)])


# Network Monitor tests
def test_change_network_monitor_to_trustee(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=NETWORK_MONITOR,
                                                                  new_value=TRUSTEE,
                                                                  is_owner=is_owner)])


def test_change_network_monitor_to_steward(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=NETWORK_MONITOR,
                                                                  new_value=STEWARD,
                                                                  is_owner=is_owner)])


def test_change_network_monitor_to_endorser(write_request_validation, req, is_owner):
    authorized = req.identifier in ("trustee_identifier", "steward_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=NETWORK_MONITOR,
                                                                  new_value=ENDORSER,
                                                                  is_owner=is_owner)])


def test_change_network_monitor_to_network_monitor(write_request_validation, req, is_owner):
    authorized = is_owner
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=NETWORK_MONITOR,
                                                                  new_value=NETWORK_MONITOR,
                                                                  is_owner=is_owner)])


def test_change_network_monitor_to_identity_owner(write_request_validation, req, is_owner):
    authorized = req.identifier in ("trustee_identifier", "steward_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value=NETWORK_MONITOR,
                                                                  new_value='',
                                                                  is_owner=is_owner)])


# Identity Owner tests
def test_change_identity_owner_to_trustee(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value='',
                                                                  new_value=TRUSTEE,
                                                                  is_owner=is_owner)])


def test_change_identity_owner_to_steward(write_request_validation, req, is_owner):
    authorized = (req.identifier == "trustee_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value='',
                                                                  new_value=STEWARD,
                                                                  is_owner=is_owner)])


def test_change_identity_owner_to_endorser(write_request_validation, req, is_owner):
    authorized = req.identifier in ("trustee_identifier", "steward_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value='',
                                                                  new_value=ENDORSER,
                                                                  is_owner=is_owner)])


def test_change_identity_owner_to_network_monitor(write_request_validation, req, is_owner):
    authorized = req.identifier in ("trustee_identifier", "steward_identifier")
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value='',
                                                                  new_value=NETWORK_MONITOR,
                                                                  is_owner=is_owner)])


def test_change_identity_owner_to_identity_owner(write_request_validation, req, is_owner):
    authorized = is_owner
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=ROLE,
                                                                  old_value='',
                                                                  new_value='',
                                                                  is_owner=is_owner)])


def test_change_verkey(write_request_validation, req, is_owner):
    authorized = is_owner
    assert authorized == write_request_validation(req,
                                                  [AuthActionEdit(txn_type=NYM,
                                                                  field=VERKEY,
                                                                  old_value="_verkey {}".format(req.identifier),
                                                                  new_value='new_value',
                                                                  is_owner=is_owner)])



"""
indy-node package metadata
"""
import os
import json
from typing import Any
import collections.abc

from indy_common.node_version import NodeVersion, InvalidVersionError

VERSION_FILENAME = '__version__.json'
VERSION_FILE = os.path.join(
    os.path.abspath(os.path.dirname(__file__)), VERSION_FILENAME)


MANIFEST_FILENAME = '__manifest__.json'
MANIFEST_FILE = os.path.join(
    os.path.abspath(os.path.dirname(__file__)), MANIFEST_FILENAME)


# TODO use/wrap plenum's set and load API
def load_version(version_file: str = VERSION_FILE) -> NodeVersion:
    with open(version_file, 'r') as _f:
        version = json.load(_f)
        if not isinstance(version, collections.abc.Iterable):
            raise InvalidVersionError(
                "Failed to load from {}: data '{}' is not iterable"
                .format(version_file, version)
            )
        return NodeVersion('.'.join([str(i) for i in version if str(i)]))


def set_version(version: str, version_file: str = VERSION_FILE):
    version = NodeVersion(version)
    with open(version_file, 'w') as _f:
        json.dump(['' if i is None else i for i in version.parts], _f)
        _f.write('\n')


def load_manifest(manifest_file: str = MANIFEST_FILE) -> Any:
    try:
        with open(manifest_file, 'r') as _f:
            return json.load(_f)
    except IOError:
        return None


def set_manifest(manifest: Any, manifest_file: str = MANIFEST_FILE):
    with open(manifest_file, 'w') as _f:
        json.dump(manifest, _f, sort_keys=True)
        _f.write('\n')


__title__ = 'indy-node'
__version_info__ = load_version()
__version__ = __version_info__.full
__manifest__ = load_manifest()
__description__ = 'Indy node'
__long_description__ = __description__
__keywords__ = 'Indy Node'
__url__ = 'https://github.com/hyperledger/indy-node'
__author__ = "Hyperledger"
__author_email__ = 'hyperledger-indy@lists.hyperledger.org'
__maintainer__ = "Hyperledger"
__license__ = "Apache 2.0"


__all__ = [
    '__title__', '__version_info__', '__version__', '__manifest__',
    '__description__', '__long_description__', '__keywords__',
    '__url__', '__author__', '__author_email__', '__maintainer__',
    '__license__', 'load_version', 'set_version', 'load_manifest', 'set_manifest']



import asyncio
from datetime import datetime

from stp_core.loop.eventually import eventuallySoon

from indy_node.server.upgrade_log import UpgradeLogData
from indy_node.server.upgrader import Upgrader, UpgradeMessage

from indy_node.test.upgrade.helper import bumpedVersion


async def _createServer(host, port):
    """
    Create async server that listens host:port, reads client request and puts
    value to some future that can be used then for checks

    :return: reference to server and future for request
    """
    indicator = asyncio.Future()

    async def _handle(reader, writer):
        raw = await reader.readline()
        request = raw.decode("utf-8")
        indicator.set_result(request)
    server = await asyncio.start_server(_handle, host, port)
    return server, indicator


def _stopServer(server):
    def _stop(x):
        print('Closing server')
        server.close()
    return _stop


def _checkFuture(future):
    """
    Wrapper for futures that lets checking of their status using 'eventually'
    """
    def _check():
        if future.cancelled():
            return None
        if future.done():
            return future.result()
        raise Exception()
    return _check


def testScheduleNodeUpgrade(tconf, nodeSet):
    """
    Tests that upgrade scheduling works. For that it starts mock
    control service, schedules upgrade for near future and then checks that
    service received notification.
    """
    loop = asyncio.get_event_loop()
    server, indicator = loop.run_until_complete(
        _createServer(
            host=tconf.controlServiceHost,
            port=tconf.controlServicePort
        )
    )
    indicator.add_done_callback(_stopServer(server))

    node = nodeSet[0]

    # ATTENTION! nodeId and ledger must not be None, but there
    # we do not call methods that use them, so we can pass None
    # We do it because node from nodeSet is some testable object, not real
    # node, so it has no nodeId and ledger that we can use
    upgrader = Upgrader(nodeId=None,
                        nodeName=None,
                        dataDir=node.dataLocation,
                        config=tconf,
                        ledger=None)
    version = bumpedVersion()
    ev_data = UpgradeLogData(
        datetime.utcnow(), version, 'some_id', tconf.UPGRADE_ENTRY)
    upgrader._callUpgradeAgent(ev_data, 1000)

    result = loop.run_until_complete(eventuallySoon(_checkFuture(indicator)))
    expectedResult = UpgradeMessage(version, tconf.UPGRADE_ENTRY)
    assert result == expectedResult.toJson()


# def testCancelNodeUpgrade():
#     """
#     Test cancellation of scheduled upgrades. This test schedules upgrade for
#     unreachable moment in future and then tries to cancel that
#     before timeout
#     """
#     loop = asyncio.get_event_loop()
#     farMomentInFuture = time.time() + 10000
#     up = node_control_tools.scheduleNodeUpgrade(farMomentInFuture, version)
#     upgrade = loop.run_until_complete(up)  # type: asyncio.Future
#     node_control_tools.cancelNodeUpgrade(version)
#     loop.run_until_complete(eventuallySoon(_checkFuture(upgrade)))



from indy_common.constants import NODE_UPGRADE, ACTION, COMPLETE, POOL_UPGRADE, START, SCHEDULE
from plenum.common.constants import VERSION, DATA
from plenum.common.txn_util import get_type, get_payload_data, get_from, get_txn_time
from plenum.common.util import SortedDict
from plenum.server.txn_version_controller import TxnVersionController as TVController


class TxnVersionController(TVController):

    def __init__(self) -> None:
        self._versions = SortedDict()
        self._f = 0
        self._votes_for_new_version = SortedDict()

    @property
    def version(self):
        return self._versions.peekitem(-1)[1] if self._versions else None

    def get_pool_version(self, timestamp):
        if timestamp is None:
            return self.version
        last_version = None
        for upgrade_tm, version in self._versions.items():
            if timestamp < upgrade_tm:
                return last_version
            last_version = version
        return last_version

    def update_version(self, txn):
        if get_type(txn) == POOL_UPGRADE and get_payload_data(txn).get(ACTION) == START:
            N = len(get_payload_data(txn).get(SCHEDULE, {}))
            self._f = (N - 1) // 3
        elif get_type(txn) == NODE_UPGRADE and get_payload_data(txn)[DATA][ACTION] == COMPLETE:
            version = get_payload_data(txn)[DATA][VERSION]
            self._votes_for_new_version.setdefault(version, set())
            self._votes_for_new_version[version].add(get_from(txn))
            if len(self._votes_for_new_version[version]) > self._f:
                self._versions[get_txn_time(txn)] = version
                self._votes_for_new_version = SortedDict({v: senders
                                                          for v, senders in self._votes_for_new_version.items()
                                                          if v > version})



from indy_node.test.pool_config.helper import check_pool_config_writable_set, \
    sdk_ensure_pool_config_sent, sdk_pool_config_sent
from stp_core.loop.eventually import eventually

from plenum.test.pool_transactions.helper import disconnect_node_and_ensure_disconnected


def test_send_pool_config_writes_false_force_false(
        nodeSet, looper, sdk_pool_handle, sdk_wallet_trustee, poolConfigWFFF):
    sdk_ensure_pool_config_sent(looper, sdk_pool_handle, sdk_wallet_trustee,
                                poolConfigWFFF)
    check_pool_config_writable_set(nodeSet, False)


def test_send_pool_config_writes_true_force_true(
        nodeSet, looper, sdk_pool_handle, sdk_wallet_trustee, poolConfigWTFT):
    sdk_ensure_pool_config_sent(looper, sdk_pool_handle, sdk_wallet_trustee,
                                poolConfigWTFT)
    check_pool_config_writable_set(nodeSet, True)


def test_send_pool_config_writes_false_force_true(
        nodeSet, looper, sdk_pool_handle, sdk_wallet_trustee, poolConfigWFFT):
    sdk_ensure_pool_config_sent(looper, sdk_pool_handle, sdk_wallet_trustee,
                                poolConfigWFFT)
    check_pool_config_writable_set(nodeSet, False)


def test_send_pool_config_writes_true_force_false(
        nodeSet, looper, sdk_pool_handle, sdk_wallet_trustee, poolConfigWTFF):
    sdk_ensure_pool_config_sent(looper, sdk_pool_handle, sdk_wallet_trustee,
                                poolConfigWTFF)
    check_pool_config_writable_set(nodeSet, True)


def test_send_pool_config_2_nodes_can_force_writes_false_force_true(
        nodeSet, looper, sdk_pool_handle, sdk_wallet_trustee, poolConfigWFFT):
    assert len(nodeSet) == 4

    node1 = nodeSet[2]
    node0 = nodeSet[3]

    check_pool_config_writable_set(nodeSet, True)

    disconnect_node_and_ensure_disconnected(
        looper, nodeSet, node0.name, stopNode=False)
    disconnect_node_and_ensure_disconnected(
        looper, nodeSet, node1.name, stopNode=False)

    sdk_pool_config_sent(looper, sdk_pool_handle,
                         sdk_wallet_trustee, poolConfigWFFT)

    looper.run(eventually(check_pool_config_writable_set,
                          nodeSet[0:2], False, retryWait=1, timeout=10))



import glob
import os
from shutil import copyfile, copy2

from ledger.genesis_txn.genesis_txn_file_util import genesis_txn_file

from indy_common.constants import Environment


class Setup:

    def __init__(self, basedir):
        self.base_dir = basedir

    def setupCommon(self):
        self.setupTxns("poolLedger")

    def setupNode(self):
        self.setupTxns("domainLedger")

    def setupClient(self):
        self.setupSampleInvites()

    def setupTxns(self, key, force: bool = False):
        """
        Create base transactions

        :param key: ledger
        :param force: replace existing transaction files
        """
        import data
        dataDir = os.path.dirname(data.__file__)

        # TODO: Need to get "test" and "live" from ENVS property in config.py
        # but that gives error due to some dependency issue
        allEnvs = {
            "local": Environment("pool_transactions_local",
                                 "domain_transactions_local"),
            "test": Environment("pool_transactions_sandbox",
                                "domain_transactions_sandbox"),
            "live": Environment("pool_transactions_live",
                                "domain_transactions_live")
        }
        for env in allEnvs.values():
            fileName = getattr(env, key, None)
            if not fileName:
                continue
            sourceFilePath = os.path.join(dataDir, fileName)
            if not os.path.exists(sourceFilePath):
                continue
            destFilePath = os.path.join(
                self.base_dir, genesis_txn_file(fileName))
            if os.path.exists(destFilePath) and not force:
                continue
            copyfile(sourceFilePath, destFilePath)

        return self

    def setupSampleInvites(self):
        import sample
        sdir = os.path.dirname(sample.__file__)
        sidir = os.path.join(self.base_dir, "sample")
        os.makedirs(sidir, exist_ok=True)
        files = glob.iglob(os.path.join(sdir, "*.indy"))
        for file in files:
            if os.path.isfile(file):
                copy2(file, sidir)
        return self



import copy
import json

import pytest

from indy_common.constants import RS_CONTENT, ENDORSER, RS_ID, RS_CRED_DEF_SCHEMA, RS_CRED_DEF_MAPPING
from indy_node.server.request_handlers.domain_req_handlers.rich_schema.rich_schema_cred_def_handler import \
    RichSchemaCredDefHandler
from indy_node.server.request_handlers.domain_req_handlers.rich_schema.rich_schema_handler import RichSchemaHandler
from indy_node.server.request_handlers.domain_req_handlers.rich_schema.rich_schema_mapping_handler import \
    RichSchemaMappingHandler
from indy_node.test.request_handlers.helper import add_to_idr
from indy_node.test.request_handlers.rich_schema.helper import rich_schema_request, rich_schema_cred_def_request, \
    rich_schema_mapping_request, make_rich_schema_object_exist
from plenum.common.constants import TRUSTEE
from plenum.common.exceptions import InvalidClientRequest
from plenum.common.util import randomString


@pytest.fixture()
def cred_def_handler(db_manager, write_auth_req_validator):
    return RichSchemaCredDefHandler(db_manager, write_auth_req_validator)


@pytest.fixture()
def rich_schema_handler(db_manager, write_auth_req_validator):
    return RichSchemaHandler(db_manager, write_auth_req_validator)


@pytest.fixture()
def mapping_handler(db_manager, write_auth_req_validator):
    return RichSchemaMappingHandler(db_manager, write_auth_req_validator)


@pytest.fixture()
def rich_schema_req():
    return rich_schema_request()


@pytest.fixture()
def mapping_req():
    return rich_schema_mapping_request()


@pytest.fixture()
def cred_def_req(rich_schema_handler, mapping_handler, rich_schema_req, mapping_req):
    make_rich_schema_object_exist(rich_schema_handler, rich_schema_req)
    make_rich_schema_object_exist(mapping_handler, mapping_req)

    req = rich_schema_cred_def_request()

    content = copy.deepcopy(json.loads(req.operation[RS_CONTENT]))
    content[RS_CRED_DEF_SCHEMA] = rich_schema_req.operation[RS_ID]
    content[RS_CRED_DEF_MAPPING] = mapping_req.operation[RS_ID]
    req.operation[RS_CONTENT] = json.dumps(content)

    add_to_idr(rich_schema_handler.database_manager.idr_cache, req.identifier, TRUSTEE)
    add_to_idr(rich_schema_handler.database_manager.idr_cache, req.endorser, ENDORSER)

    return req


def test_static_validation_pass(cred_def_handler, cred_def_req):
    cred_def_handler.static_validation(cred_def_req)


@pytest.mark.parametrize('missing_field', ['signatureType', 'mapping', 'schema', 'publicKey'])
@pytest.mark.parametrize('status', ['missing', 'empty', 'none'])
def test_static_validation_no_field(cred_def_handler, cred_def_req, missing_field, status):
    content = copy.deepcopy(json.loads(cred_def_req.operation[RS_CONTENT]))
    if status == 'missing':
        content.pop(missing_field, None)
    elif status == 'empty':
        content[missing_field] = ""
    elif status == 'none':
        content[missing_field] = None
    cred_def_req.operation[RS_CONTENT] = json.dumps(content)

    with pytest.raises(InvalidClientRequest,
                       match="'{}' must be set in 'content'".format(missing_field)):
        cred_def_handler.static_validation(cred_def_req)


@pytest.mark.parametrize('status', ['missing', 'empty', 'none'])
def test_static_validation_no_all_fields(cred_def_handler, cred_def_req, status):
    content = copy.deepcopy(json.loads(cred_def_req.operation[RS_CONTENT]))
    if status == 'missing':
        content.pop('signatureType', None)
        content.pop('mapping', None)
        content.pop('schema', None)
        content.pop('publicKey', None)
    elif status == 'empty':
        content['signatureType'] = ""
        content['mapping'] = ""
        content['schema'] = ""
        content['publicKey'] = ""
    elif status == 'none':
        content['signatureType'] = None
        content['mapping'] = None
        content['schema'] = None
        content['publicKey'] = None
    cred_def_req.operation[RS_CONTENT] = json.dumps(content)

    with pytest.raises(InvalidClientRequest,
                       match="'signatureType', 'mapping', 'schema', 'publicKey' must be set in 'content'"):
        cred_def_handler.static_validation(cred_def_req)

def test_dynamic_validation_passes(cred_def_handler, cred_def_req):
    cred_def_handler.dynamic_validation(cred_def_req, 0)


@pytest.mark.parametrize('field', [RS_CRED_DEF_SCHEMA, RS_CRED_DEF_MAPPING])
def test_dynamic_validation_not_existent_ref(cred_def_handler, cred_def_req,
                                             field):
    content = copy.deepcopy(json.loads(cred_def_req.operation[RS_CONTENT]))
    wrong_id = randomString()
    content[field] = wrong_id
    cred_def_req.operation[RS_CONTENT] = json.dumps(content)

    with pytest.raises(InvalidClientRequest,
                       match="Can not find a referenced '{}' with id={}; please make sure that it has been added to the ledger".format(
                           field, wrong_id)):
        cred_def_handler.dynamic_validation(cred_def_req, 0)


def test_dynamic_validation_not_schema_in_schema_field(cred_def_handler, cred_def_req,
                                                       mapping_req):
    content = copy.deepcopy(json.loads(cred_def_req.operation[RS_CONTENT]))
    content[RS_CRED_DEF_SCHEMA] = mapping_req.operation[RS_ID]
    cred_def_req.operation[RS_CONTENT] = json.dumps(content)

    with pytest.raises(InvalidClientRequest,
                       match="'schema' field must reference a schema with rsType=sch"):
        cred_def_handler.dynamic_validation(cred_def_req, 0)


def test_dynamic_validation_not_mapping_in_mapping_field(cred_def_handler, cred_def_req,
                                                         rich_schema_req):
    content = copy.deepcopy(json.loads(cred_def_req.operation[RS_CONTENT]))
    content[RS_CRED_DEF_MAPPING] = rich_schema_req.operation[RS_ID]
    cred_def_req.operation[RS_CONTENT] = json.dumps(content)

    with pytest.raises(InvalidClientRequest,
                       match="'mapping' field must reference a mapping with rsType=map"):
        cred_def_handler.dynamic_validation(cred_def_req, 0)



import os
import pytest
import logging

from indy_common import strict_types
from indy_common.config import GENERAL_CONFIG_FILE
from indy_common.config_util import getConfig
from indy_common.txn_util import getTxnOrderedFields
from indy_node.general_config import ubuntu_platform_config, windows_platform_config, general_config
from indy_node.server.config_helper import create_config_dirs
from stp_core.common.log import getlogger
from indy_common.config_helper import ConfigHelper, NodeConfigHelper

# typecheck during tests
strict_types.defaultShouldCheck = True

# noinspection PyUnresolvedReferences
from plenum.test.conftest import GENERAL_CONFIG_DIR, \
    overriddenConfigValues  # noqa

logger = getlogger()


@pytest.fixture(scope="module")
def config_helper_class():
    return ConfigHelper


@pytest.fixture(scope="module")
def node_config_helper_class():
    return NodeConfigHelper


def _populate_config_file(config_file, platform='ubuntu'):
    if platform == 'ubuntu':
        platform_config = ubuntu_platform_config
    elif platform == 'windows':
        platform_config = windows_platform_config
    else:
        raise RuntimeError('Unknown platform {}. Cannot load config'.format(platform))

    with open(general_config.__file__, 'r') as general_config_file:
        lines = general_config_file.readlines()
        with open(platform_config.__file__, 'r') as platform_config_file:
            with open(config_file,
                      'w') as general_config_result_file:
                for line in lines:
                    if line.startswith('NETWORK_NAME'):
                        line = 'NETWORK_NAME = \'sandbox\'\n'
                    general_config_result_file.write(line)
                general_config_result_file.write(
                    platform_config_file.read())


def _general_conf_tdir(tmp_dir):
    cdir = create_config_dirs(tmp_dir)
    config_file = os.path.join(cdir, GENERAL_CONFIG_FILE)
    _populate_config_file(config_file)
    return cdir


@pytest.fixture(scope='module')
def general_conf_tdir(tdir):
    general_config_dir = _general_conf_tdir(tdir)
    logger.debug("module-level general config directory: {}".format(general_config_dir))
    return general_config_dir


@pytest.fixture()
def general_conf_tdir_for_func(tdir_for_func):
    general_config_dir = _general_conf_tdir(tdir_for_func)
    logger.debug("function-level general config directory: {}".format(general_config_dir))
    return general_config_dir


def _tconf(general_config):
    config = getConfig(general_config_dir=general_config)
    for k, v in overriddenConfigValues.items():
        setattr(config, k, v)
    config.MinSepBetweenNodeUpgrades = 5
    config.RETRY_TIMEOUT_RESTRICTED = 15
    config.RETRY_TIMEOUT_NOT_RESTRICTED = 15
    config.MAX_RECONNECT_RETRY_ON_SAME_SOCKET = 1
    return config


@pytest.fixture(scope="module")
def tconf(general_conf_tdir):
    return _tconf(general_conf_tdir)


@pytest.fixture()
def tconf_for_func(general_conf_tdir_for_func):
    return _tconf(general_conf_tdir_for_func)


@pytest.fixture(scope="module")
def poolTxnTrusteeNames():
    return "Trustee1",


@pytest.fixture(scope="module")  # noqa
def looper(txnPoolNodesLooper):
    return txnPoolNodesLooper


@pytest.fixture(scope="module")
def domainTxnOrderedFields():
    return getTxnOrderedFields()


@pytest.fixture(autouse=True)
def setTestLogLevel():
    logger = getlogger()
    logger.level = logging.NOTSET



import pytest

from indy_common.constants import REVOC_REG_ENTRY, REVOC_REG_DEF_ID, ISSUANCE_BY_DEFAULT, \
    VALUE, ISSUANCE_TYPE, ISSUED, REVOKED, ACCUM
from indy_node.server.request_handlers.domain_req_handlers.revoc_reg_def_handler import RevocRegDefHandler
from indy_node.server.request_handlers.domain_req_handlers.revoc_reg_entry_handler import RevocRegEntryHandler
from indy_node.test.request_handlers.helper import add_to_idr
from plenum.common.constants import TXN_TIME, TRUSTEE

from plenum.common.exceptions import InvalidClientRequest, UnauthorizedClientRequest
from plenum.common.request import Request
from plenum.common.txn_util import reqToTxn, append_txn_metadata, get_payload_data
from plenum.common.types import f
from plenum.common.util import randomString
from plenum.server.request_handlers.utils import encode_state_value


@pytest.fixture(scope="module")
def revoc_reg_entry_handler(db_manager, write_auth_req_validator):
    return RevocRegEntryHandler(db_manager, write_auth_req_validator,
                                RevocRegDefHandler.get_revocation_strategy)


@pytest.fixture(scope="function")
def revoc_reg_entry_request():
    identifier = randomString()
    return Request(identifier= identifier,
                   reqId=5,
                   operation={'type': REVOC_REG_ENTRY,
                              REVOC_REG_DEF_ID: identifier,
                              VALUE: {ACCUM: 5}},
                   signature="randomString")


def test_revoc_reg_entry_dynamic_validation_without_req_def(revoc_reg_entry_handler,
                                                            revoc_reg_entry_request):
    with pytest.raises(InvalidClientRequest,
                       match="There is no any REVOC_REG_DEF by path"):
        revoc_reg_entry_handler.dynamic_validation(revoc_reg_entry_request, 0)


def test_revoc_reg_entry_dynamic_validation_passes(revoc_reg_entry_handler,
                                                   revoc_reg_entry_request):
    add_to_idr(revoc_reg_entry_handler.database_manager.idr_cache,
               revoc_reg_entry_request.identifier,
               TRUSTEE)

    revoc_reg_entry_handler.state.set(revoc_reg_entry_request.operation[REVOC_REG_DEF_ID].encode(),
                                      encode_state_value({VALUE: {ISSUANCE_TYPE: ISSUANCE_BY_DEFAULT}},
                                                         "seqNo", "txnTime"))
    revoc_reg_entry_handler.dynamic_validation(revoc_reg_entry_request, 0)


def test_revoc_reg_entry_dynamic_validation_fail_in_strategy(revoc_reg_entry_handler,
                                                             revoc_reg_entry_request):
    add_to_idr(revoc_reg_entry_handler.database_manager.idr_cache,
               revoc_reg_entry_request.identifier,
               TRUSTEE)
    revoc_reg_entry_handler.state.set(revoc_reg_entry_request.operation[REVOC_REG_DEF_ID].encode(),
                                      encode_state_value({VALUE: {ISSUANCE_TYPE: ISSUANCE_BY_DEFAULT}},
                                                         "seqNo", "txnTime"))
    revoc_reg_entry_request.operation[VALUE] = {ISSUED: [1],
                                                REVOKED: [1]}
    with pytest.raises(InvalidClientRequest, match="Can not have an index in both "
                                                   "'issued' and 'revoked' lists"):
        revoc_reg_entry_handler.dynamic_validation(revoc_reg_entry_request, 0)


def test_revoc_reg_entry_dynamic_validation_without_permission(revoc_reg_entry_handler,
                                                               revoc_reg_entry_request):
    add_to_idr(revoc_reg_entry_handler.database_manager.idr_cache,
               revoc_reg_entry_request.identifier,
               None)
    revoc_reg_entry_handler.state.set(revoc_reg_entry_request.operation[REVOC_REG_DEF_ID].encode(),
                                      encode_state_value({VALUE: {ISSUANCE_TYPE: ISSUANCE_BY_DEFAULT}},
                                                         "seqNo", "txnTime"))
    revoc_reg_entry_request.operation[VALUE] = {ISSUED: [1],
                                                REVOKED: [1]}
    with pytest.raises(UnauthorizedClientRequest, match="1 TRUSTEE signature is required and needs to be owner OR "
                                                        "1 STEWARD signature is required and needs to be owner OR "
                                                        "1 ENDORSER signature is required and needs to be owner"):
        revoc_reg_entry_handler.dynamic_validation(revoc_reg_entry_request, 0)


def test_failed_update_state(revoc_reg_entry_handler, revoc_reg_entry_request):
    seq_no = 1
    txn_time = 1560241033
    txn = reqToTxn(revoc_reg_entry_request)
    append_txn_metadata(txn, seq_no, txn_time)
    with pytest.raises(InvalidClientRequest,
                       match="There is no any REVOC_REG_DEF by path"):
        revoc_reg_entry_handler.update_state(txn, None, revoc_reg_entry_request)


def test_update_state(revoc_reg_entry_handler, revoc_reg_entry_request,
                      revoc_reg_def_handler, revoc_reg_def_request):
    # create revoc_req_def
    seq_no = 1
    txn_time = 1560241030
    revoc_reg_def_request.operation[VALUE] = {}
    revoc_reg_def_request.operation[VALUE][ISSUANCE_TYPE] = ISSUANCE_BY_DEFAULT
    txn = reqToTxn(revoc_reg_def_request)
    append_txn_metadata(txn, seq_no, txn_time)
    path = RevocRegDefHandler.prepare_revoc_def_for_state(txn,
                                                          path_only=True)
    revoc_reg_def_handler.update_state(txn, None, revoc_reg_def_request)

    # create revoc_req_entry
    seq_no = 2
    txn_time = 1560241033
    revoc_reg_entry_request.operation[REVOC_REG_DEF_ID] = path.decode()
    txn = reqToTxn(revoc_reg_entry_request)
    append_txn_metadata(txn, seq_no, txn_time)
    revoc_reg_entry_handler.update_state(txn, None, revoc_reg_entry_request)

    # check state for revoc_reg_entry
    txn_data = get_payload_data(txn)
    txn_data[f.SEQ_NO.nm] = seq_no
    txn_data[TXN_TIME] = txn_time
    assert revoc_reg_entry_handler.get_from_state(
        RevocRegEntryHandler.prepare_revoc_reg_entry_for_state(txn,
                                                               path_only=True)) == (txn_data, seq_no, txn_time)
    # check state for revoc_reg_entry
    txn_data[VALUE] = {ACCUM: txn_data[VALUE][ACCUM]}
    path, _ = RevocRegEntryHandler.prepare_revoc_reg_entry_accum_for_state(txn)
    assert revoc_reg_entry_handler.get_from_state(path) == (txn_data, seq_no, txn_time)



import pytest
import json

from indy.did import abbreviate_verkey, create_and_store_my_did
from indy.ledger import build_get_nym_request, build_nym_request

from plenum.common.util import randomString
from plenum.test.helper import sdk_get_and_check_replies
from plenum.test.pool_transactions.helper import sdk_sign_and_send_prepared_request


def add_new_nym(looper, sdk_pool_handle, creators_wallet, dest, verkey=None):
    wh, submitter_did = creators_wallet
    nym_request = looper.loop.run_until_complete(build_nym_request(submitter_did, dest, verkey, None, None))
    req = sdk_sign_and_send_prepared_request(looper, creators_wallet, sdk_pool_handle, nym_request)
    sdk_get_and_check_replies(looper, [req])
    return wh, dest


@pytest.fixture(scope='function')
def nym_full_vk(looper, tconf, nodeSet, sdk_pool_handle, sdk_wallet_trustee):
    wh, tr_did = sdk_wallet_trustee
    (new_did, new_verkey) = looper.loop.run_until_complete(
        create_and_store_my_did(wh, json.dumps({'seed': randomString(32)})))
    assert not new_verkey.startswith("~")
    nwh, nd = add_new_nym(looper, sdk_pool_handle, sdk_wallet_trustee, dest=new_did, verkey=new_verkey)
    return nwh, nd, new_verkey


@pytest.fixture(scope='function')
def nym_abbrv_vk(looper, tconf, nodeSet, sdk_pool_handle, sdk_wallet_trustee):
    wh, tr_did = sdk_wallet_trustee
    (new_did, new_verkey) = looper.loop.run_until_complete(
        create_and_store_my_did(wh, json.dumps({'seed': randomString(32)})))
    abbrv_vk = looper.loop.run_until_complete(abbreviate_verkey(new_did, new_verkey))
    assert abbrv_vk.startswith("~")
    nwh, nd = add_new_nym(looper, sdk_pool_handle, sdk_wallet_trustee, dest=new_did, verkey=abbrv_vk)
    return nwh, nd, abbrv_vk


@pytest.fixture(scope='function')
def nym_empty_vk(looper, tconf, nodeSet, sdk_pool_handle, sdk_wallet_trustee):
    wh, tr_did = sdk_wallet_trustee
    (new_did, new_verkey) = looper.loop.run_until_complete(
        create_and_store_my_did(wh, json.dumps({'seed': randomString(32)})))
    nwh, nd = add_new_nym(looper, sdk_pool_handle, sdk_wallet_trustee, dest=new_did)
    return nwh, nd


def nym_get(looper, sdk_pool_handle, sdk_wallet_sign, did):
    sign_w, sign_did = sdk_wallet_sign
    get_nym_req = looper.loop.run_until_complete(build_get_nym_request(sign_did, did))
    req = sdk_sign_and_send_prepared_request(looper, sdk_wallet_sign, sdk_pool_handle, get_nym_req)
    repl_data = sdk_get_and_check_replies(looper, [req])[0][1].get("result", {}).get("data", "")
    dt = json.loads(repl_data)
    nym = dt.get("dest", None)
    vk = dt.get("verkey", None)
    return nym, vk



from typing import Optional

from indy_common.authorize.auth_actions import AuthActionEdit
from indy_common.authorize.auth_request_validator import WriteRequestValidator
from indy_common.constants import AUTH_RULES, RULES
from indy_node.server.request_handlers.config_req_handlers.auth_rule.abstract_auth_rule_handler import \
    AbstractAuthRuleHandler
from indy_node.server.request_handlers.config_req_handlers.auth_rule.static_auth_rule_helper import StaticAuthRuleHelper
from plenum.common.request import Request
from plenum.common.txn_util import get_payload_data
from plenum.server.database_manager import DatabaseManager


class AuthRulesHandler(AbstractAuthRuleHandler):

    def __init__(self, database_manager: DatabaseManager, write_req_validator: WriteRequestValidator):
        super().__init__(database_manager, write_req_validator, AUTH_RULES)

    def static_validation(self, request: Request):
        identifier, req_id, operation = request.identifier, request.reqId, request.operation
        self._validate_request_type(request)
        for rule in operation.get(RULES):
            self._static_validation_for_rule(rule, identifier, req_id)

    def additional_dynamic_validation(self, request: Request, req_pp_time: Optional[int]):
        self._validate_request_type(request)
        self.write_req_validator.validate(request,
                                          [AuthActionEdit(txn_type=AUTH_RULES,
                                                          field="*",
                                                          old_value="*",
                                                          new_value="*")])

    def update_state(self, txn, prev_result, request=None, is_committed=False):
        payload = get_payload_data(txn)
        for rule in payload.get(RULES, []):
            constraint = StaticAuthRuleHelper.get_auth_constraint(rule)
            auth_key = StaticAuthRuleHelper.get_auth_key(rule)
            self._update_auth_constraint(auth_key, constraint)



import json

from indy_node.test.helper import createHalfKeyIdentifierAndAbbrevVerkey
from indy.ledger import sign_request, submit_request, build_nym_request
from plenum.common.constants import REPLY, REJECT


def test_nym_send_twice(looper, sdk_pool_handle, sdk_wallet_steward):
    idr, verkey = createHalfKeyIdentifierAndAbbrevVerkey()

    wallet_handle, identifier = sdk_wallet_steward

    for i in range(2):
        request = looper.loop.run_until_complete(build_nym_request(identifier, idr, verkey, None, None))
        req_signed = looper.loop.run_until_complete(sign_request(wallet_handle, identifier, request))
        result = json.loads(looper.loop.run_until_complete(submit_request(sdk_pool_handle, req_signed)))

        if i == 0:
            assert result['op'] == REPLY
        else:
            assert result['op'] == REJECT


def test_nym_resend(looper, sdk_pool_handle, sdk_wallet_steward):
    idr, verkey = createHalfKeyIdentifierAndAbbrevVerkey()

    wallet_handle, identifier = sdk_wallet_steward

    request = looper.loop.run_until_complete(build_nym_request(identifier, idr, verkey, None, None))
    req_signed = looper.loop.run_until_complete(sign_request(wallet_handle, identifier, request))

    for i in range(2):
        result = json.loads(looper.loop.run_until_complete(submit_request(sdk_pool_handle, req_signed)))
        assert result['op'] == REPLY



from copy import deepcopy
from hashlib import sha256

from plenum.common.types import OPERATION
from plenum.common.constants import TXN_TYPE, RAW, ENC, HASH, NYM
from plenum.server.client_authn import NaclAuthNr, CoreAuthMixin

from indy_common.constants import ATTRIB
from indy_node.persistence.idr_cache import IdrCache
from plenum.server.request_handlers.utils import get_request_type, nym_ident_is_dest, get_target_verkey


class LedgerBasedAuthNr(CoreAuthMixin, NaclAuthNr):
    """
    Transaction-based client authenticator.
    """

    def __init__(self, write_types, query_types, action_types, cache: IdrCache):
        NaclAuthNr.__init__(self)
        CoreAuthMixin.__init__(self, write_types, query_types, action_types)
        self.cache = cache
        self.specific_verkey_validation = {NYM: self.nym_specific_auth}

    def serializeForSig(self, msg, identifier=None, topLevelKeysToIgnore=None):
        if msg[OPERATION].get(TXN_TYPE) == ATTRIB:
            msgCopy = deepcopy(msg)
            keyName = {RAW, ENC, HASH}.intersection(
                set(msgCopy[OPERATION].keys())).pop()
            msgCopy[OPERATION][keyName] = sha256(
                msgCopy[OPERATION][keyName].encode()).hexdigest()
            return super().serializeForSig(msgCopy, identifier=identifier,
                                           topLevelKeysToIgnore=topLevelKeysToIgnore)
        else:
            return super().serializeForSig(msg, identifier=identifier,
                                           topLevelKeysToIgnore=topLevelKeysToIgnore)

    def addIdr(self, identifier, verkey, role=None):
        raise RuntimeError('Add verification keys through the NYM txn')

    def getVerkey(self, ident, request):
        try:
            verkey = self.cache.getVerkey(ident, isCommitted=False)
        except KeyError:
            # If DID wasn't found in idr cache, then it can be
            # creation of a new DID signed by a verkey for this new DID
            verkey = self.get_verkey_specific(request)
            if not verkey:
                return None
            else:
                return verkey
        return verkey

    def get_verkey_specific(self, request):
        typ = get_request_type(request)
        verkey_validation = self.specific_verkey_validation.get(typ)
        if verkey_validation is None:
            return None
        verkey = verkey_validation(request)
        return verkey

    def nym_specific_auth(self, request):
        # As far as we allow non-ledger nyms send their own txn,
        # we need to check if it's target verkey is correct
        if nym_ident_is_dest(request):
            return get_target_verkey(request)
        else:
            return None



# DOMAIN LEDGER
MARKER_ATTR = "1"
MARKER_SCHEMA = "2"
MARKER_CLAIM_DEF = "3"
MARKER_REVOC_DEF = "4"
MARKER_REVOC_REG_ENTRY = "5"
MARKER_REVOC_REG_ENTRY_ACCUM = "6"
MARKER_CONTEXT = "7"
MARKER_RS_SCHEMA = "8"
LAST_SEQ_NO = "lsn"
VALUE = "val"
LAST_UPDATE_TIME = "lut"

# CONFIG LEDGER
MARKER_AUTH_RULE = "1"



from indy_common.state.domain import make_state_path_for_attr, make_state_path_for_schema, \
    make_state_path_for_claim_def, make_state_path_for_revoc_def, make_state_path_for_revoc_reg_entry, \
    make_state_path_for_revoc_reg_entry_accum


def test_make_state_path_for_attr():
    assert b'did1:1:attrName1' == make_state_path_for_attr('did1', 'attrName1', attr_is_hash=True)
    assert b'did1:1:677a81e8649df8f1a1e8af7709a5ece1d965cb684b2c185272114c5cc3b7ec49' == \
           make_state_path_for_attr('did1', 'attrName1', attr_is_hash=False)
    assert b'did1:1:677a81e8649df8f1a1e8af7709a5ece1d965cb684b2c185272114c5cc3b7ec49' == \
           make_state_path_for_attr('did1', 'attrName1')


def test_make_state_path_for_schema():
    assert b'did1:2:name1:version1' == make_state_path_for_schema('did1', 'name1', 'version1')


def test_make_state_path_for_claim_def():
    assert b'did1:3:CL:18:tag' == make_state_path_for_claim_def('did1', 18, 'CL', 'tag')


def test_make_state_path_for_revoc_def():
    assert b'did1:4:did1:3:18:CL:tag:CL_ACCUM:tag' == \
           make_state_path_for_revoc_def('did1', 'did1:3:18:CL:tag', 'CL_ACCUM', 'tag')


def test_make_state_path_for_revoc_reg_entry():
    assert b'5:did1:4:did1:3:18:CL:tag:CL_ACCUM:tag' == \
           make_state_path_for_revoc_reg_entry('did1:4:did1:3:18:CL:tag:CL_ACCUM:tag')


def test_make_state_path_for_revoc_reg_entry_accum():
    assert b'6:did1:4:did1:3:18:CL:tag:CL_ACCUM:tag' == \
           make_state_path_for_revoc_reg_entry_accum('did1:4:did1:3:18:CL:tag:CL_ACCUM:tag')


def test_prepare_attr_for_state():
    pass



from indy_common.authorize.auth_actions import AuthActionAdd
from indy_common.authorize.auth_request_validator import WriteRequestValidator
from plenum.server.database_manager import DatabaseManager
from plenum.server.request_handlers.txn_author_agreement_aml_handler import TxnAuthorAgreementAmlHandler \
    as PTxnAuthorAgreementAmlHandler


class TxnAuthorAgreementAmlHandler(PTxnAuthorAgreementAmlHandler):

    def __init__(self, database_manager: DatabaseManager,
                 write_req_validator: WriteRequestValidator):
        super().__init__(database_manager)
        self.write_req_validator = write_req_validator

    def authorize(self, request):
        self.write_req_validator.validate(request,
                                          [AuthActionAdd(txn_type=self.txn_type,
                                                         field='*',
                                                         value='*')])




from typing import Any, Optional, Tuple, Union, cast
from plenum.common.txn_util import get_txn_time
from plenum.server.database_manager import DatabaseManager
from plenum.server.request_handlers.handler_interfaces.read_request_handler import ReadRequestHandler
from plenum.server.node import Node
from storage.state_ts_store import StateTsDbStorage

from indy_common.constants import DOMAIN_LEDGER_ID


class VersionReadRequestHandler(ReadRequestHandler):
    """Specialized read request handler enabling looking up past versions."""

    def __init__(
        self, node: Node, database_manager: DatabaseManager, txn_type, ledger_id
    ):
        super().__init__(database_manager, txn_type, ledger_id)
        self.node = node
        self.timestamp_store: StateTsDbStorage = cast(
            StateTsDbStorage, self.database_manager.ts_store
        )

    def lookup_version(
        self,
        path: bytes,
        seq_no: Optional[int] = None,
        timestamp: Optional[int] = None,
        with_proof=False,
    ) -> Tuple[Optional[Union[bytes, str]], Any]:
        """Lookup a value from the ledger state, optionally retrieving from the past.

        If seq_no or timestamp is specified and no value is found, returns (None, None).
        If neither are specified, value in its current state is retrieved, which will
        also return (None, None) if it is not found on the ledger.

        seq_no and timestamp are mutually exclusive.
        """
        if seq_no is not None and timestamp is not None:
            raise ValueError("seq_no and timestamp are mutually exclusive")
        # The above check determines whether the method is used correctly
        # A similar check in GetNymHandler and GetAttributeHandler determines
        # whether the request is valid

        if seq_no:
            timestamp = self._timestamp_from_seq_no(seq_no)
            if not timestamp:
                return None, None

        if timestamp:
            past_root = self.timestamp_store.get_equal_or_prev(timestamp)
            if past_root:
                return self._get_value_from_state(
                    path, head_hash=past_root, with_proof=with_proof
                )

            return None, None

        return self._get_value_from_state(
            path, with_proof=with_proof
        )

    def _timestamp_from_seq_no(self, seq_no: int) -> Optional[int]:
        """Return timestamp of a transaction identified by seq_no."""
        db = self.database_manager.get_database(DOMAIN_LEDGER_ID)
        txn = self.node.getReplyFromLedger(db.ledger, seq_no, write=False)

        if txn and "result" in txn:
            return get_txn_time(txn.result)
        return None



from typing import NamedTuple

from abc import ABCMeta, abstractmethod

RULE_DELIMETER = "--"
ADD_PREFIX = "ADD"
EDIT_PREFIX = "EDIT"

ActionDef = NamedTuple('ActionDef',
                       [('txn_type', str), ('prefix', str), ('field', str), ('old_value', str), ('new_value', str)])


def compile_action_id(txn_type,
                      field,
                      old_value,
                      new_value,
                      prefix='') -> str:
    return RULE_DELIMETER.join([txn_type,
                                prefix,
                                field,
                                old_value,
                                new_value])


def split_action_id(action_id) -> ActionDef:
    return ActionDef(*action_id.split(RULE_DELIMETER))


"""

Action's classes

"""


class AbstractAuthAction(metaclass=ABCMeta):
    def __init__(self, txn_type):
        pass

    @abstractmethod
    def get_action_id(self) -> str:
        raise NotImplementedError()


class AuthActionAdd(AbstractAuthAction):
    def __init__(self,
                 txn_type,
                 field=None,
                 value=None,
                 is_owner: bool = True):
        self.txn_type = txn_type
        self.field = str(field) if field is not None else ''
        self.value = str(value) if value is not None else ''
        self.is_owner = is_owner

    def get_action_id(self) -> str:
        return compile_action_id(txn_type=self.txn_type,
                                 field=self.field,
                                 old_value='*',
                                 new_value=self.value,
                                 prefix=ADD_PREFIX)


class AuthActionEdit(AbstractAuthAction):
    def __init__(self,
                 txn_type,
                 field=None,
                 old_value=None,
                 new_value=None,
                 is_owner: bool = True):
        self.txn_type = txn_type
        self.field = str(field) if field is not None else ''
        self.old_value = str(old_value) if old_value is not None else ''
        self.new_value = str(new_value) if new_value is not None else ''
        self.is_owner = is_owner

    def get_action_id(self) -> str:
        return compile_action_id(txn_type=self.txn_type,
                                 field=self.field,
                                 old_value=self.old_value,
                                 new_value=self.new_value,
                                 prefix=EDIT_PREFIX)



import copy
import json
import time
from random import randint

import pytest
from indy_common.constants import NYM_VERSION_CONVENTION
from indy_node.test.helper import sdk_send_and_check_req_json
from indy_node.test.mock import build_get_nym_request, build_nym_request
from plenum.common.exceptions import RequestNackedException
from plenum.test.helper import sdk_get_and_check_replies
from plenum.test.pool_transactions.helper import sdk_sign_and_send_prepared_request


def test_get_nym_data_with_diddoc_content_without_seq_no_or_timestamp(
    looper, sdk_pool_handle, sdk_wallet_endorser_factory, diddoc_content, diddoc_content_json
):
    sdk_wallet_endorser = sdk_wallet_endorser_factory(diddoc_content)
    _, did = sdk_wallet_endorser
    get_nym_request = build_get_nym_request(did, did)
    request_couple = sdk_sign_and_send_prepared_request(
        looper, sdk_wallet_endorser, sdk_pool_handle, get_nym_request
    )
    replies = sdk_get_and_check_replies(looper, [request_couple])

    assert (
        json.loads(replies[0][1]["result"]["data"])["diddocContent"] == diddoc_content_json
    )


def test_get_previous_nym_data_by_timestamp(
    looper, sdk_pool_handle, sdk_wallet_endorser_factory, diddoc_content, diddoc_content_json
):
    sdk_wallet_endorser = sdk_wallet_endorser_factory(diddoc_content)
    _, did = sdk_wallet_endorser

    # Get current nym data
    get_nym_request = build_get_nym_request(did, did)
    request_couple = sdk_sign_and_send_prepared_request(
        looper, sdk_wallet_endorser, sdk_pool_handle, get_nym_request
    )
    replies = sdk_get_and_check_replies(looper, [request_couple])

    # Get timestamp from data
    timestamp = replies[0][1]["result"]["txnTime"]

    # Write new nym data
    new_diddoc_content = copy.deepcopy(diddoc_content)
    new_diddoc_content["serviceEndpoint"][0][
        "serviceEndpoint"
    ] = "https://new.example.com"
    new_diddoc_content = json.dumps(new_diddoc_content)

    time.sleep(3)

    nym_request = build_nym_request(
        identifier=did, dest=did, diddoc_content=new_diddoc_content
    )
    request_couple = sdk_sign_and_send_prepared_request(
        looper, sdk_wallet_endorser, sdk_pool_handle, nym_request
    )
    sdk_get_and_check_replies(looper, [request_couple])

    get_nym_request = build_get_nym_request(did, did)
    request_couple = sdk_sign_and_send_prepared_request(
        looper, sdk_wallet_endorser, sdk_pool_handle, get_nym_request
    )
    replies = sdk_get_and_check_replies(looper, [request_couple])

    assert (
        json.loads(replies[0][1]["result"]["data"])["diddocContent"] == new_diddoc_content
    )

    update_ts = replies[0][1]["result"]["txnTime"]

    # Get previous nym data by exact timestamp
    get_nym_request = build_get_nym_request(did, did, timestamp)
    request_couple = sdk_sign_and_send_prepared_request(
        looper, sdk_wallet_endorser, sdk_pool_handle, get_nym_request
    )
    replies = sdk_get_and_check_replies(looper, [request_couple])

    assert (
        json.loads(replies[0][1]["result"]["data"])["diddocContent"] == diddoc_content_json
    )

    # Get previous nym data by timestamp but not exact
    ts = randint(timestamp + 1, update_ts - 1)
    get_nym_request = build_get_nym_request(did, did, ts)
    request_couple = sdk_sign_and_send_prepared_request(
        looper, sdk_wallet_endorser, sdk_pool_handle, get_nym_request
    )
    replies = sdk_get_and_check_replies(looper, [request_couple])

    assert (
        json.loads(replies[0][1]["result"]["data"])["diddocContent"] == diddoc_content_json
    )


def test_get_previous_nym_data_by_seq_no(
    looper, sdk_pool_handle, sdk_wallet_endorser_factory, diddoc_content, diddoc_content_json
):
    sdk_wallet_endorser = sdk_wallet_endorser_factory(diddoc_content)
    _, did = sdk_wallet_endorser

    # Get current nym data
    get_nym_request = build_get_nym_request(did, did)
    request_couple = sdk_sign_and_send_prepared_request(
        looper, sdk_wallet_endorser, sdk_pool_handle, get_nym_request
    )
    replies = sdk_get_and_check_replies(looper, [request_couple])

    # Get seq_no from data
    seq_no = replies[0][1]["result"]["seqNo"]

    # Write new nym data
    new_diddoc_content = copy.deepcopy(diddoc_content)
    new_diddoc_content["serviceEndpoint"][0][
        "serviceEndpoint"
    ] = "https://new.example.com"
    new_diddoc_content = json.dumps(new_diddoc_content)

    time.sleep(3)

    nym_request = build_nym_request(did, did, None, new_diddoc_content, None)
    request_couple = sdk_sign_and_send_prepared_request(
        looper, sdk_wallet_endorser, sdk_pool_handle, nym_request
    )
    sdk_get_and_check_replies(looper, [request_couple])

    get_nym_request = build_get_nym_request(did, did)
    request_couple = sdk_sign_and_send_prepared_request(
        looper, sdk_wallet_endorser, sdk_pool_handle, get_nym_request
    )
    replies = sdk_get_and_check_replies(looper, [request_couple])

    assert (
        json.loads(replies[0][1]["result"]["data"])["diddocContent"] == new_diddoc_content
    )

    # Get previous nym data by seq_no
    get_nym_request = build_get_nym_request(did, did, seq_no=seq_no)
    request_couple = sdk_sign_and_send_prepared_request(
        looper, sdk_wallet_endorser, sdk_pool_handle, get_nym_request
    )
    replies = sdk_get_and_check_replies(looper, [request_couple])

    assert (
        json.loads(replies[0][1]["result"]["data"])["diddocContent"] == diddoc_content_json
    )


def test_nym_txn_rejected_with_both_seqNo_and_timestamp(
    looper, sdk_pool_handle, sdk_wallet_endorser
):
    _, did = sdk_wallet_endorser

    # Attempt to get previous nym data by exact timestamp and seqNo
    get_nym_request = build_get_nym_request(did, did, int(time.time()), 10)

    with pytest.raises(RequestNackedException) as e:
        sdk_send_and_check_req_json(
            looper, sdk_pool_handle, sdk_wallet_endorser, get_nym_request
        )
    e.match("InvalidClientRequest")
    e.match("client request invalid")
    e.match("mutually exclusive")


def test_get_nym_returns_no_nym_version_when_absent(
    looper, sdk_pool_handle, sdk_wallet_endorser
):
    _, did = sdk_wallet_endorser
    get_nym_request = build_get_nym_request(did, did)
    request_couple = sdk_sign_and_send_prepared_request(
        looper, sdk_wallet_endorser, sdk_pool_handle, get_nym_request
    )
    replies = sdk_get_and_check_replies(looper, [request_couple])

    assert (
        "version" not in json.loads(replies[0][1]["result"]["data"])
    )


def test_get_nym_returns_nym_version_when_set(
    looper, sdk_pool_handle, sdk_wallet_endorser_factory
):
    sdk_wallet_endorser = sdk_wallet_endorser_factory(version=NYM_VERSION_CONVENTION)
    _, did = sdk_wallet_endorser
    get_nym_request = build_get_nym_request(did, did)
    request_couple = sdk_sign_and_send_prepared_request(
        looper, sdk_wallet_endorser, sdk_pool_handle, get_nym_request
    )
    replies = sdk_get_and_check_replies(looper, [request_couple])

    returned = json.loads(replies[0][1]["result"]["data"])
    assert "version" in returned
    assert returned["version"] == NYM_VERSION_CONVENTION



#!/usr/bin/python3.5
import pwd
import grp
import os
import stat
import shutil
from importlib.util import module_from_spec, spec_from_file_location

from stp_core.common.log import getlogger

baseDir = '/var/lib/indy'
GENERAL_CONFIG_DIR = '/etc/indy'
GENERAL_CONFIG_FILE = 'indy_config.py'
LOG_DIR = '/var/log/indy'
NODE_BASE_DATA_DIR = baseDir
CLI_BASE_DIR = '/home/indy/.indy-cli'
CLI_NETWORK_DIR = '/home/indy/.indy-cli/networks'

logger = getlogger()

old_base_dir = '/home/indy/.indy'


def ext_copytree(src, dst):
    if not os.path.isdir(dst):
        shutil.copytree(src, dst)


def migrate_genesis_txn(old_base_txn_dir, new_base_txn_dir, new_cli_net_base_dir):
    logger.info('Move genesis transactions {} -> {}'.format(old_base_txn_dir, new_base_txn_dir))
    for suffix in ('sandbox', 'live', 'local'):
        new_txn_dir = os.path.join(new_base_txn_dir, suffix)
        new_cli_network_dir = os.path.join(new_cli_net_base_dir, suffix)
        os.makedirs(new_txn_dir, exist_ok=True)
        os.makedirs(new_cli_network_dir, exist_ok=True)

        old_domain_genesis = os.path.join(old_base_txn_dir, 'domain_transactions_{}_genesis'.format(suffix))
        old_pool_genesis = os.path.join(old_base_txn_dir, 'pool_transactions_{}_genesis'.format(suffix))
        new_domain_genesis = os.path.join(new_txn_dir, 'domain_transactions_genesis')
        new_pool_genesis = os.path.join(new_txn_dir, 'pool_transactions_genesis')
        new_cli_domain_genesis = os.path.join(new_cli_network_dir, 'domain_transactions_genesis')
        new_cli_pool_genesis = os.path.join(new_cli_network_dir, 'pool_transactions_genesis')

        if os.path.exists(old_domain_genesis):
            shutil.copy2(old_domain_genesis, new_domain_genesis)
            shutil.copy2(old_domain_genesis, new_cli_domain_genesis)
        if os.path.exists(old_pool_genesis):
            shutil.copy2(old_pool_genesis, new_pool_genesis)
            shutil.copy2(old_pool_genesis, new_cli_pool_genesis)

    logger.info('done')


def _get_network_from_txn_file_name(file_name: str):
    name_split = file_name.split("_")
    ret_name = "sandbox"
    if name_split and name_split[-1] in ["live", "local", "sandbox"]:
        ret_name = name_split[-1]
    return ret_name


def get_network_name():
    network_name = 'sandbox'
    old_general_config = os.path.join(old_base_dir, 'indy_config.py')
    spec = spec_from_file_location('old_general_config', old_general_config)
    old_cfg = module_from_spec(spec)
    spec.loader.exec_module(old_cfg)
    if hasattr(old_cfg, 'poolTransactionsFile'):
        network_name = _get_network_from_txn_file_name(old_cfg.poolTransactionsFile)
    elif hasattr(old_cfg, 'domainTransactionsFile'):
        network_name = _get_network_from_txn_file_name(old_cfg.domainTransactionsFile)
    elif hasattr(old_cfg, 'current_env') and old_cfg.current_env != 'test':
        network_name = old_cfg.current_env
    return network_name


def migrate_general_config(old_general_config, new_general_config, network_name):
    logger.info('Migrate general config file {} -> {} for network {}'.format(
        old_general_config, new_general_config, network_name))

    f = open(old_general_config, "r")
    lines = f.readlines()
    f.close()

    f = open(new_general_config, "a+")
    f.write("\n")
    for line in lines:
        if not line.startswith('current_env') and not line.startswith('poolTransactionsFile') and\
                not line.startswith('domainTransactionsFile'):
            f.write(line)
    line = "NETWORK_NAME = '{}'\n".format(network_name)
    f.write(line)
    f.close()
    # os.remove(old_general_config)


def remove_network_from_dir_name(root_dir):
    try:
        visit_dirs = os.listdir(root_dir)
    except FileNotFoundError:
        visit_dirs = []
    for dname in visit_dirs:
        _dname = dname
        _dname = _dname.replace("_sandbox", "")
        _dname = _dname.replace("_live", "")
        _dname = _dname.replace("_local", "")
        if _dname != dname:
            os.rename(os.path.join(root_dir, dname), os.path.join(root_dir, _dname))
        if _dname in ['pool_state', 'domain_state', 'config_state', 'idr_cache_db']:
            shutil.rmtree(os.path.join(root_dir, _dname))


def set_own_perm(usr, dir_list):
    uid = pwd.getpwnam(usr).pw_uid
    gid = grp.getgrnam(usr).gr_gid
    perm_mask_rw = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IWGRP
    perm_mask_rwx = stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IXGRP

    for cdir in dir_list:
        os.chown(cdir, uid, gid)
        os.chmod(cdir, perm_mask_rwx)
        for croot, sub_dirs, cfiles in os.walk(cdir):
            for fs_name in sub_dirs:
                os.chown(os.path.join(croot, fs_name), uid, gid)
                os.chmod(os.path.join(croot, fs_name), perm_mask_rwx)
            for fs_name in cfiles:
                os.chown(os.path.join(croot, fs_name), uid, gid)
                os.chmod(os.path.join(croot, fs_name), perm_mask_rw)


def migrate_all():
    # Check which network name should we use
    network_name = get_network_name()
    logger.info('Network name is {}'.format(network_name))
    base_dir = baseDir

    logger.info('Start migration of directories/files.')

    # Migrate general config file
    old_general_config = os.path.join(old_base_dir, 'indy_config.py')
    new_general_config = os.path.join(GENERAL_CONFIG_DIR, GENERAL_CONFIG_FILE)
    os.makedirs(GENERAL_CONFIG_DIR, exist_ok=True)
    migrate_general_config(old_general_config, new_general_config, network_name)
    old_srv_cfg = os.path.join(old_base_dir, 'indy.env')
    new_srv_cfg = os.path.join(GENERAL_CONFIG_DIR, 'indy.env')
    shutil.copy2(old_srv_cfg, new_srv_cfg)

    # Create indy cli folder
    os.makedirs(CLI_NETWORK_DIR, exist_ok=True)

    # Move genesis transactions
    migrate_genesis_txn(old_base_dir, NODE_BASE_DATA_DIR, CLI_NETWORK_DIR)

    # Move data
    old_nodes_data_dir = os.path.join(old_base_dir, 'data', 'nodes')
    new_node_data_dir = os.path.join(NODE_BASE_DATA_DIR, network_name, 'data', 'nodes')
    try:
        visit_dirs = os.listdir(old_nodes_data_dir)
    except FileNotFoundError:
        visit_dirs = []
    for node_name in visit_dirs:
        move_path = os.path.join(old_nodes_data_dir, node_name)
        to_path = os.path.join(new_node_data_dir, node_name)
        ext_copytree(move_path, to_path)
        remove_network_from_dir_name(to_path)
    # shutil.rmtree(os.path.join(old_base_dir, 'data', 'nodes'))

    # Move *_info* files and logs; delete system __pycache__; move key folder
    new_log_dir = os.path.join(LOG_DIR, network_name)
    os.makedirs(new_log_dir, exist_ok=True)
    new_key_dir = os.path.join(base_dir, network_name, 'keys')
    os.makedirs(new_key_dir, exist_ok=True)
    try:
        visit_dirs = os.listdir(old_base_dir)
    except FileNotFoundError:
        visit_dirs = []
    for fs_name in visit_dirs:
        full_path = os.path.join(old_base_dir, fs_name)
        if "_info" in fs_name:
            shutil.copy2(full_path, os.path.join(base_dir, network_name))
        elif ".log" in fs_name:
            shutil.copy2(full_path, new_log_dir)
        elif os.path.isdir(full_path) and fs_name not in ['data', '__pycache__', 'wallets', 'sample', 'plugins']:
            ext_copytree(full_path, os.path.join(new_key_dir, fs_name))

    # Remove old dir
    # os.removedirs(old_base_dir)

    logger.info('Finish migration of directories/files.')


if not os.path.exists(old_base_dir):
    msg = 'Can not find old base directory \'{}\'. Nothing to migrate.'.format(old_base_dir)
    logger.warning(msg)


migrate_all()
set_own_perm("indy", ["/etc/indy", "/var/lib/indy", "/var/log/indy", "/home/indy/.indy-cli"])



import os
from datetime import datetime
from functools import partial
from typing import Union

import dateutil.parser
import dateutil.tz

from indy_common.types import Request
from indy_node.server.node_maintainer import NodeMaintainer, \
    NodeControlToolMessage
from indy_node.server.restart_log import RestartLogData, RestartLog
from stp_core.common.log import getlogger
from plenum.common.constants import TXN_TYPE
from indy_common.constants import ACTION, POOL_RESTART, START, DATETIME, \
    CANCEL, TIMEOUT, RESTART_MESSAGE
import asyncio

logger = getlogger()


class Restarter(NodeMaintainer):

    def _defaultLog(self, dataDir, config):
        log = os.path.join(dataDir, config.restartLogFile)
        return RestartLog(file_path=log)

    def _is_action_started(self):
        last_action = self.lastActionEventInfo
        if not last_action:
            logger.debug('Node {} has no restart events'
                         .format(self.nodeName))
            return False

        if last_action.ev_type != RestartLog.Events.started:
            logger.info(
                "Restart for node {} was not scheduled. Last event is {}"
                .format(self.nodeName, last_action))
            return False

        return True

    def _update_action_log_for_started_action(self):
        self._actionLog.append_succeeded(self.lastActionEventInfo.data)
        logger.info("Node '{}' successfully restarted"
                    .format(self.nodeName))

    def handleRestartRequest(self, req: Request) -> None:
        """
        Handles transaction of type POOL_RESTART
        Can schedule or cancel restart to a newer
        version at specified time

        :param req:
        """
        txn = req.operation
        if txn[TXN_TYPE] != POOL_RESTART:
            return

        action = txn[ACTION]
        if action == START:
            when = dateutil.parser.parse(txn[DATETIME]) \
                if DATETIME in txn.keys() and txn[DATETIME] not in ["0", "", None] \
                else None
            fail_timeout = txn.get(TIMEOUT, self.defaultActionTimeout)
            self.requestRestart(when, fail_timeout)
            return

        if action == CANCEL:
            if self.scheduledAction:
                self._cancelScheduledRestart()
                logger.info("Node '{}' cancels restart".format(
                    self.nodeName))
            return

        logger.error(
            "Got {} transaction with unsupported action {}".format(
                POOL_RESTART, action))

    def requestRestart(self, when=None, fail_timeout=None):
        if self.scheduledAction:
            if self.scheduledAction.when == when:
                logger.debug(
                    "Node {} already scheduled restart".format(
                        self.nodeName))
                return
            else:
                logger.info(
                    "Node '{}' cancels previous restart and schedules a new one".format(
                        self.nodeName))
                self._cancelScheduledRestart()

        now = datetime.utcnow().replace(tzinfo=dateutil.tz.tzutc())
        if when is None or now >= when:
            msg = RestartMessage().toJson()
            try:
                asyncio.ensure_future(self._open_connection_and_send(msg))
            except Exception as ex:
                logger.warning(ex.args[0])
            return

        if fail_timeout is None:
            fail_timeout = self.defaultActionTimeout
        logger.info("Node '{}' schedules restart".format(
            self.nodeName))

        self._scheduleRestart(when, fail_timeout)

    def _scheduleRestart(self,
                         when: Union[datetime, str],
                         failTimeout) -> None:
        """
        Schedules node restart to a newer version

        :param version: version to restart to
        :param when: restart time
        """
        logger.info("{}'s restarter processing restart"
                    .format(self))
        if isinstance(when, str):
            when = dateutil.parser.parse(when)
        now = datetime.utcnow().replace(tzinfo=dateutil.tz.tzutc())

        logger.info(
            "Restart of node '{}' has been scheduled on {}".format(
                self.nodeName, when))

        ev_data = RestartLogData(when)
        self._actionLog.append_scheduled(ev_data)

        callAgent = partial(self._callRestartAgent, ev_data,
                            failTimeout)
        delay = 0
        if now < when:
            delay = (when - now).total_seconds()
        self.scheduledAction = ev_data
        self._schedule(callAgent, delay)

    def _cancelScheduledRestart(self, justification=None) -> None:
        """
        Cancels scheduled restart

        :param when: time restart was scheduled to
        :param version: version restart scheduled for
        """

        if self.scheduledAction:
            why_prefix = ": "
            why = justification
            if justification is None:
                why_prefix = ", "
                why = "cancellation reason not specified"

            ev_data = self.scheduledAction
            logger.info("Cancelling restart"
                        " of node {}"
                        " scheduled on {}"
                        "{}{}"
                        .format(self.nodeName,
                                ev_data.when,
                                why_prefix,
                                why))

            self._unscheduleAction()
            self._actionLog.append_cancelled(ev_data)
            logger.info(
                "Restart of node '{}'"
                "has been cancelled due to {}".format(
                    self.nodeName, why))

    def _callRestartAgent(self, ev_data: RestartLogData, failTimeout) -> None:
        """
        Callback which is called when restart time come.
        Writes restart record to restart log and asks
        node control service to perform restart

        :param ev_data: restart event data
        :param version: version to restart to
        """

        logger.info("{}'s restart calling agent for restart".format(self))
        self._actionLog.append_started(ev_data)
        self._action_start_callback()
        self.scheduledAction = None
        asyncio.ensure_future(
            self._sendUpdateRequest(ev_data, failTimeout))

    async def _sendUpdateRequest(self, ev_data: RestartLogData, failTimeout):
        retryLimit = self.retry_limit
        while retryLimit:
            try:
                msg = RestartMessage().toJson()
                logger.info("Sending message to control tool: {}".format(msg))
                await self._open_connection_and_send(msg)
                break
            except Exception as ex:
                logger.warning("Failed to communicate to control tool: {}"
                               .format(ex))
                asyncio.sleep(self.retry_timeout)
                retryLimit -= 1
        if not retryLimit:
            self._action_failed(ev_data,
                                reason="problems in communication with "
                                       "node control service")
            self._unscheduleAction()
            self._actionFailedCallback()
        else:
            logger.info("Waiting {} minutes for restart to be performed"
                        .format(failTimeout))
            timesUp = partial(self._declareTimeoutExceeded, ev_data)
            self._schedule(timesUp, self.get_timeout(failTimeout))

    def _declareTimeoutExceeded(self, ev_data: RestartLogData):
        """
        This function is called when time for restart is up
        """
        logger.info("Timeout exceeded for {}".format(ev_data.when))
        last = self._actionLog.last_event
        if (last and last.ev_type == RestartLog.Events.failed and last.data == ev_data):
            return None

        self._action_failed(ev_data,
                            reason="exceeded restart timeout")

        self._unscheduleAction()
        self._actionFailedCallback()

    def _action_failed(self, *,
                       ev_data: RestartLogData,
                       reason=None,
                       external_reason=False):
        if reason is None:
            reason = "unknown reason"
        error_message = "Node {} failed restart" \
                        "scheduled on {} " \
                        "because of {}" \
            .format(self.nodeName,
                    ev_data.when,
                    reason)
        logger.error(error_message)
        if external_reason:
            logger.error("This problem may have external reasons, "
                         "check syslog for more information")


class RestartMessage(NodeControlToolMessage):
    """
    Data structure that represents request for node update
    """

    def __init__(self):
        super().__init__(RESTART_MESSAGE)

    def toJson(self):
        import json
        return json.dumps(self.__dict__)



from collections import OrderedDict
from indy_common.types import RevocDefValueField, ClientRevocDefSubmitField
from plenum.common.messages.fields import IntegerField, AnyMapField, NonEmptyStringField, ConstantField, ChooseField, \
    LimitedLengthStringField
from indy_common.constants import ISSUANCE_TYPE, MAX_CRED_NUM, PUBLIC_KEYS, \
    TAILS_LOCATION, TAILS_HASH, ID, REVOC_TYPE, TXN_TYPE, TAG, CRED_DEF_ID, VALUE


EXPECTED_REVOC_DEF_VALUE_FIELDS = OrderedDict([
    (ISSUANCE_TYPE, ChooseField),
    (MAX_CRED_NUM, IntegerField),
    (PUBLIC_KEYS, AnyMapField),
    (TAILS_HASH, NonEmptyStringField),
    (TAILS_LOCATION, NonEmptyStringField),
])

EXPECTED_REVOC_DEF_SUBMIT_FIELDS = OrderedDict([
    (TXN_TYPE, ConstantField),
    (ID, NonEmptyStringField),
    (REVOC_TYPE, LimitedLengthStringField),
    (TAG, LimitedLengthStringField),
    (CRED_DEF_ID, NonEmptyStringField),
    (VALUE, RevocDefValueField)
])


def test_revoc_value_has_expected_fields():
    actual_field_names = OrderedDict(
        RevocDefValueField.schema).keys()
    assert actual_field_names == EXPECTED_REVOC_DEF_VALUE_FIELDS.keys()


def test_revoc_value_has_expected_validators():
    schema = dict(RevocDefValueField.schema)
    for field, validator in EXPECTED_REVOC_DEF_VALUE_FIELDS.items():
        assert isinstance(schema[field], validator)


def test_client_submit_has_expected_fields():
    actual_field_names = OrderedDict(
        ClientRevocDefSubmitField.schema).keys()
    assert actual_field_names == EXPECTED_REVOC_DEF_SUBMIT_FIELDS.keys()


def test_client_submit_has_expected_validators():
    schema = dict(ClientRevocDefSubmitField.schema)
    for field, validator in EXPECTED_REVOC_DEF_SUBMIT_FIELDS.items():
        assert isinstance(schema[field], validator)



#!/usr/bin/python3.5

from indy_common.migration.helper import update_indy_env

from stp_core.common.log import getlogger

logger = getlogger()

CLIENT_CONNECTIONS_LIMIT_KEY = "CLIENT_CONNECTIONS_LIMIT"
CLIENT_CONNECTIONS_LIMIT_VAL = 500


logger.info("Going to update clients connections limit.")
update_indy_env(CLIENT_CONNECTIONS_LIMIT_KEY, CLIENT_CONNECTIONS_LIMIT_VAL)
logger.info("Done.")



import pytest

from indy_common.constants import GET_ATTR, GET_NYM, GET_SCHEMA, GET_CLAIM_DEF, CLAIM_DEF_FROM, CLAIM_DEF_SCHEMA_REF, \
    CLAIM_DEF_SIGNATURE_TYPE, SCHEMA_NAME, SCHEMA_VERSION, SCHEMA_ATTR_NAMES, JSON_LD_CONTEXT, RICH_SCHEMA, \
    RICH_SCHEMA_ENCODING, RICH_SCHEMA_MAPPING, RICH_SCHEMA_CRED_DEF, RS_CRED_DEF_TYPE_VALUE, \
    RS_ENCODING_TYPE_VALUE, RS_MAPPING_TYPE_VALUE, RS_SCHEMA_TYPE_VALUE, RS_CONTEXT_TYPE_VALUE, RS_ID, \
    GET_RICH_SCHEMA_OBJECT_BY_ID, GET_RICH_SCHEMA_OBJECT_BY_METADATA, RS_NAME, RS_VERSION, RS_TYPE, \
    RICH_SCHEMA_PRES_DEF, RS_PRES_DEF_TYPE_VALUE
from indy_node.test.rich_schema.templates import W3C_BASE_CONTEXT, RICH_SCHEMA_EX1
from indy_node.test.state_proof.helper import check_valid_proof, \
    sdk_submit_operation_and_get_result
from plenum.common.constants import TARGET_NYM, TXN_TYPE, RAW, DATA
# fixtures, do not remove
from indy_node.test.attrib_txn.test_nym_attrib import \
    sdk_added_raw_attribute, attributeName, attributeValue, attributeData

from plenum.common.util import randomString


def check_no_data_and_valid_proof(result):
    assert result.get(DATA) is None
    check_valid_proof(result)


def test_state_proof_returned_for_missing_attr(looper, nodeSetWithOneNodeResponding,
                                               attributeName,
                                               sdk_pool_handle,
                                               sdk_wallet_endorser):
    """
    Tests that state proof is returned in the reply for GET_ATTR transactions
    """
    _, dest = sdk_wallet_endorser

    get_attr_operation = {
        TARGET_NYM: dest,
        TXN_TYPE: GET_ATTR,
        RAW: attributeName
    }
    result = sdk_submit_operation_and_get_result(looper, sdk_pool_handle,
                                                 sdk_wallet_endorser, get_attr_operation)
    check_no_data_and_valid_proof(result)


def test_state_proof_returned_for_missing_nym(looper, nodeSetWithOneNodeResponding,
                                              sdk_pool_handle,
                                              sdk_wallet_endorser,
                                              sdk_user_wallet_a):
    """
    Tests that state proof is returned in the reply for GET_NYM transactions
    """
    # Make not existing id
    _, dest = sdk_user_wallet_a
    dest = dest[:-3]
    dest += "fff"

    get_nym_operation = {
        TARGET_NYM: dest,
        TXN_TYPE: GET_NYM
    }

    result = sdk_submit_operation_and_get_result(looper, sdk_pool_handle,
                                                 sdk_wallet_endorser, get_nym_operation)
    check_no_data_and_valid_proof(result)


def test_state_proof_returned_for_missing_schema(looper, nodeSetWithOneNodeResponding,
                                                 sdk_pool_handle,
                                                 sdk_wallet_endorser):
    """
    Tests that state proof is returned in the reply for GET_SCHEMA transactions
    """
    _, dest = sdk_wallet_endorser
    schema_name = "test_schema"
    schema_version = "1.0"
    get_schema_operation = {
        TARGET_NYM: dest,
        TXN_TYPE: GET_SCHEMA,
        DATA: {
            SCHEMA_NAME: schema_name,
            SCHEMA_VERSION: schema_version,
        }
    }
    result = sdk_submit_operation_and_get_result(looper, sdk_pool_handle,
                                                 sdk_wallet_endorser,
                                                 get_schema_operation)
    assert SCHEMA_ATTR_NAMES not in result[DATA]
    check_valid_proof(result)


def test_state_proof_returned_for_missing_claim_def(looper, nodeSetWithOneNodeResponding,
                                                    sdk_pool_handle,
                                                    sdk_wallet_endorser):
    """
    Tests that state proof is returned in the reply for GET_CLAIM_DEF
    transactions
    """
    _, dest = sdk_wallet_endorser
    get_claim_def_operation = {
        CLAIM_DEF_FROM: dest,
        TXN_TYPE: GET_CLAIM_DEF,
        CLAIM_DEF_SCHEMA_REF: 12,
        CLAIM_DEF_SIGNATURE_TYPE: 'CL'
    }
    result = sdk_submit_operation_and_get_result(looper, sdk_pool_handle,
                                                 sdk_wallet_endorser,
                                                 get_claim_def_operation)
    check_no_data_and_valid_proof(result)


@pytest.mark.skip
# TODO fix this test so it does not rely on Indy-SDK,
# or, fix this test once Rich Schema objects are part of Indy-SDK
def test_state_proof_returned_for_missing_get_rich_schema_obj_by_id(looper,
                                                                    nodeSetWithOneNodeResponding,
                                                                    sdk_wallet_endorser,
                                                                    sdk_pool_handle,
                                                                    sdk_wallet_client):
    """
    Tests that state proof is returned in the reply for GET_RICH_SCHEMA_OBJECT_BY_ID.
    Use different submitter and reader!
    """
    rs_id = randomString()
    get_rich_schema_by_id_operation = {
        TXN_TYPE: GET_RICH_SCHEMA_OBJECT_BY_ID,
        RS_ID: rs_id,
    }

    result = sdk_submit_operation_and_get_result(looper, sdk_pool_handle,
                                                 sdk_wallet_client,
                                                 get_rich_schema_by_id_operation)
    check_no_data_and_valid_proof(result)


@pytest.mark.skip
# TODO fix this test so it does not rely on Indy-SDK,
# or, fix this test once Rich Schewma objects are part of Indy-SDK
@pytest.mark.parametrize('rs_type',
                         [RS_CONTEXT_TYPE_VALUE, RS_SCHEMA_TYPE_VALUE, RS_ENCODING_TYPE_VALUE, RS_MAPPING_TYPE_VALUE,
                          RS_CRED_DEF_TYPE_VALUE, RS_PRES_DEF_TYPE_VALUE])
def test_state_proof_returned_for_missing_get_rich_schema_obj_by_metadata(looper,
                                                                          nodeSetWithOneNodeResponding,
                                                                          sdk_wallet_endorser,
                                                                          sdk_pool_handle,
                                                                          sdk_wallet_client,
                                                                          rs_type):
    """
    Tests that state proof is returned in the reply for GET_RICH_SCHEMA_OBJECT_BY_ID.
    Use different submitter and reader!
    """
    rs_name = randomString()
    rs_version = '1.0'
    get_rich_schema_by_metadata_operation = {
        TXN_TYPE: GET_RICH_SCHEMA_OBJECT_BY_METADATA,
        RS_NAME: rs_name,
        RS_VERSION: rs_version,
        RS_TYPE: rs_type
    }
    result = sdk_submit_operation_and_get_result(looper, sdk_pool_handle,
                                                 sdk_wallet_client,
                                                 get_rich_schema_by_metadata_operation)
    check_no_data_and_valid_proof(result)



class GraphDBNotPresent(Exception):
    reason = 'Install and then configure a Graph Database'


class InvalidConnectionException(Exception):
    pass


class InvalidDIDDocException(Exception):
    def __init__(self, reason: str):
        self.reason = reason


class NotFound(RuntimeError):
    pass


class ConnectionNotFound(NotFound):
    def __init__(self, name: str = None):
        if name:
            self.reason = "Connection with name not found {}".format(name)


class VerkeyNotFound(NotFound):
    pass


class RemoteEndpointNotFound(NotFound):
    pass


class RemotePubkeyNotFound(NotFound):
    pass


class ConnectionAlreadyExists(RuntimeError):
    pass


class LinkNotReady(RuntimeError):
    """
    Some operation is attempted on a link that is not ready for that operation
    """


class NotConnectedToNetwork(RuntimeError):
    pass



from datetime import datetime

from stp_core.loop.eventually import eventually
from indy_node.server.upgrade_log import UpgradeLogData
from indy_node.server.upgrader import Upgrader
import functools

whitelist = ['Failed to upgrade node',
             'failed upgrade',
             'This problem may have external reasons, check syslog for more information']


def testTimeoutWorks(nodeSet, looper, monkeypatch, tconf):
    """
    Checks that after some timeout upgrade is marked as failed if
    it not started
    """
    async def mock(*x):
        return None

    # patch get_timeout not to wait one whole minute
    monkeypatch.setattr(Upgrader, 'get_timeout', lambda self, timeout: timeout)
    # patch _open_connection_and_send to make node think it sent upgrade
    # successfully
    monkeypatch.setattr(Upgrader, '_open_connection_and_send', mock)
    pending = {node.name for node in nodeSet}
    version = '1.5.1'
    timeout = 1

    def chk():
        nonlocal pending
        assert len(pending) == 0

    def upgrade_failed_callback_test(nodeName):
        nonlocal pending
        pending.remove(nodeName)

    ev_data = UpgradeLogData(datetime.utcnow(), version, 'some_id', tconf.UPGRADE_ENTRY)
    for node in nodeSet:
        monkeypatch.setattr(
            node.upgrader,
            '_actionFailedCallback',
            functools.partial(
                upgrade_failed_callback_test,
                node.name))
        looper.run(node.upgrader._sendUpgradeRequest(ev_data, timeout))

    looper.run(eventually(chk))



from typing import Any, Mapping

from plenum.common.constants import DOMAIN_LEDGER_ID, ENC, HASH, RAW, TARGET_NYM
from plenum.common.exceptions import InvalidClientRequest
from plenum.common.request import Request
from plenum.common.txn_util import get_request_data
from plenum.server.database_manager import DatabaseManager
from plenum.server.request_handlers.utils import decode_state_value
from stp_core.common.log import getlogger

from indy_common.constants import ATTRIB, GET_ATTR, VERSION_ID, VERSION_TIME
from indy_node.server.request_handlers.domain_req_handlers.attribute_handler import (
    AttributeHandler,
)
from indy_node.server.request_handlers.read_req_handlers.version_read_request_handler import (
    VersionReadRequestHandler,
)
from indy_node.server.request_handlers.utils import validate_attrib_keys

logger = getlogger()


class GetAttributeHandler(VersionReadRequestHandler):

    def __init__(self, node, database_manager: DatabaseManager):
        super().__init__(node, database_manager, GET_ATTR, DOMAIN_LEDGER_ID)

    def get_result(self, request: Request):
        self._validate_request_type(request)

        identifier, req_id, operation = get_request_data(request)

        timestamp = operation.get(VERSION_TIME)
        seq_no = operation.get(VERSION_ID)
        if timestamp and seq_no:
            raise InvalidClientRequest(
                request.identifier,
                request.reqId,
                f"{VERSION_ID} and {VERSION_TIME} are mutually exclusive; only one should be "
                "specified",
            )
        # The above check determines whether the request is valid
        # A similar check in VersionReadRequestHandler determines
        # whether the method is used correctly

        if not validate_attrib_keys(operation):
            raise InvalidClientRequest(
                identifier, req_id,
                '{} should have one and only one of '
                '{}, {}, {}'
                .format(ATTRIB, RAW, ENC, HASH)
            )

        attr_type = self._get_attr_type(operation)
        path = AttributeHandler.make_state_path_for_attr(
            operation[TARGET_NYM],
            operation[attr_type],
            attr_type == HASH,
        )

        encoded, proof = self.lookup_version(
            path, seq_no=seq_no, timestamp=timestamp, with_proof=True
        )

        if not encoded:
            return self.make_result(
                request=request,
                data=None,
                last_seq_no=None,
                update_time=None,
                proof=proof
            )

        store_key, last_seq_no, last_update_time = decode_state_value(encoded)

        if attr_type == HASH:
            return self.make_result(
                request=request,
                data=operation[HASH],
                last_seq_no=last_seq_no,
                update_time=last_update_time,
                proof=proof
            )

        store_value = self._get_value_from_attribute_store(store_key)
        return self.make_result(
            request=request,
            data=store_value,
            last_seq_no=last_seq_no,
            update_time=last_update_time,
            proof=proof
        )

    @staticmethod
    def _get_attr_type(operation: Mapping[str, Any]):
        """Return attribute type based on presence of keys in operation."""
        if RAW in operation:
            return RAW
        if ENC in operation:
            return ENC
        return HASH

    def _get_value_from_attribute_store(self, key: str):
        """Retrieve value from attribute store or return None if it doesn't exist."""
        try:
            value = self.database_manager.attribute_store.get(key)
        except KeyError:
            logger.error(
                'Could not get value from attribute store for %s', key
            )
            return None
        return value



#!/usr/bin/python3.5
import os
import subprocess

from stp_core.common.log import getlogger

from indy_common.util import compose_cmd
from indy_node.utils.node_control_tool import TIMEOUT

logger = getlogger()

migration_script_path = \
    os.path.normpath(
        os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'helper_1_0_96_to_1_0_97.py'))

logger.info('script path {}'.format(migration_script_path))
ret = subprocess.run(
    compose_cmd(
        ["su -c 'python3 {}' sovrin".format(migration_script_path)]
    ),
    shell=True,
    timeout=TIMEOUT)

if ret.returncode != 0:
    msg = 'Migration failed: script returned {}'.format(ret.returncode)
    logger.error(msg)
    raise Exception(msg)



from typing import Type

from common.version import (
    DigitDotVersion, SourceVersion, PackageVersion
)

from indy_common.constants import APP_NAME
from indy_common.node_version import NodeVersion


NodeVersion = NodeVersion


class SchemaVersion(DigitDotVersion):
    def __init__(self, version: str, **kwargs):
        super().__init__(version, parts_num=(2, 3), **kwargs)


class TopPkgDefVersion(DigitDotVersion, SourceVersion):
    def __init__(self, version: str, **kwargs):
        super().__init__(version, parts_num=(2, 3), **kwargs)


def src_version_cls(pkg_name: str = APP_NAME) -> Type[SourceVersion]:
    # TODO implement dynamic class resolving depending on packge name and config
    return NodeVersion if pkg_name == APP_NAME else TopPkgDefVersion



from enum import Enum, unique

from plenum.common.roles import Roles


@unique
class Roles(Enum):
    #  These numeric constants CANNOT be changed once they have been used,
    #  because that would break backwards compatibility with the ledger
    #  Also the numeric constants CANNOT collide with the roles in plenum
    IDENTITY_OWNER = Roles.IDENTITY_OWNER.value
    TRUSTEE = Roles.TRUSTEE.value
    STEWARD = Roles.STEWARD.value
    ENDORSER = "101"
    NETWORK_MONITOR = "201"

    def __str__(self):
        return self.name

    @staticmethod
    def nameFromValue(value):
        # TODO: think about a term for a user with None role
        return Roles(value).name if value else 'None role'



from copy import deepcopy
from typing import Callable, Optional

from indy_common.authorize.auth_actions import AuthActionEdit, AuthActionAdd
from indy_common.authorize.auth_request_validator import WriteRequestValidator
from indy_common.constants import REVOC_REG_ENTRY, REVOC_REG_DEF_ID, VALUE, ISSUANCE_TYPE
from indy_common.state.state_constants import MARKER_REVOC_REG_ENTRY, MARKER_REVOC_REG_ENTRY_ACCUM
from plenum.common.constants import DOMAIN_LEDGER_ID, TXN_TIME
from plenum.common.exceptions import InvalidClientRequest
from plenum.common.request import Request
from plenum.common.txn_util import get_from, get_payload_data, get_req_id, get_request_data, get_txn_time, get_seq_no
from plenum.common.types import f

from plenum.server.database_manager import DatabaseManager
from plenum.server.request_handlers.handler_interfaces.write_request_handler import WriteRequestHandler
from plenum.server.request_handlers.utils import encode_state_value


class RevocRegEntryHandler(WriteRequestHandler):

    def __init__(self, database_manager: DatabaseManager,
                 write_req_validator: WriteRequestValidator,
                 get_revocation_strategy: Callable):
        super().__init__(database_manager, REVOC_REG_ENTRY, DOMAIN_LEDGER_ID)
        self.get_revocation_strategy = get_revocation_strategy
        self.write_req_validator = write_req_validator

    def static_validation(self, request: Request):
        pass

    def additional_dynamic_validation(self, request: Request, req_pp_time: Optional[int]):
        self._validate_request_type(request)
        rev_reg_tags = request.operation[REVOC_REG_DEF_ID]
        author_did, req_id, operation = get_request_data(request)
        current_entry, revoc_def = self._get_current_revoc_entry_and_revoc_def(
            author_did=author_did,
            revoc_reg_def_id=operation[REVOC_REG_DEF_ID],
            req_id=req_id
        )
        rev_ref_def_author_did = rev_reg_tags.split(":", 1)[0]
        is_owner = rev_ref_def_author_did == author_did

        if current_entry:
            self.write_req_validator.validate(request,
                                              [AuthActionEdit(txn_type=REVOC_REG_ENTRY,
                                                              field='*',
                                                              old_value='*',
                                                              new_value='*',
                                                              is_owner=is_owner)])
        else:
            self.write_req_validator.validate(request,
                                              [AuthActionAdd(txn_type=REVOC_REG_ENTRY,
                                                             field='*',
                                                             value='*',
                                                             is_owner=is_owner)])
        validator_cls = self.get_revocation_strategy(revoc_def[VALUE][ISSUANCE_TYPE])
        validator = validator_cls(self.state)
        validator.validate(current_entry, request)

    def gen_txn_id(self, txn):
        self._validate_txn_type(txn)
        path = RevocRegEntryHandler.prepare_revoc_reg_entry_for_state(txn, path_only=True)
        return path.decode()

    def update_state(self, txn, prev_result, request, is_committed=False):
        self._validate_txn_type(txn)
        current_entry, revoc_def = self._get_current_revoc_entry_and_revoc_def(
            author_did=get_from(txn),
            revoc_reg_def_id=get_payload_data(txn)[REVOC_REG_DEF_ID],
            req_id=get_req_id(txn)
        )
        writer_cls = self.get_revocation_strategy(
            revoc_def[VALUE][ISSUANCE_TYPE])
        writer = writer_cls(self.state)
        writer.write(current_entry, txn)
        return txn

    def _get_current_revoc_entry_and_revoc_def(self, author_did, revoc_reg_def_id, req_id):
        assert revoc_reg_def_id
        current_entry, _, _ = self._get_revoc_def_entry(revoc_reg_def_id=revoc_reg_def_id)
        revoc_def, _, _ = self.get_from_state(revoc_reg_def_id)
        if revoc_def is None:
            raise InvalidClientRequest(author_did,
                                       req_id,
                                       "There is no any REVOC_REG_DEF by path: {}".format(revoc_reg_def_id))
        return current_entry, revoc_def

    def _get_revoc_def_entry(self,
                             revoc_reg_def_id) -> (str, int, int):
        assert revoc_reg_def_id
        path = RevocRegEntryHandler.make_state_path_for_revoc_reg_entry(revoc_reg_def_id=revoc_reg_def_id)
        try:
            keys, seq_no, last_update_time = self.get_from_state(path)
            return keys, seq_no, last_update_time
        except KeyError:
            return None, None, None, None

    @staticmethod
    def prepare_revoc_reg_entry_accum_for_state(txn):
        author_did = get_from(txn)
        txn_data = get_payload_data(txn)
        revoc_reg_def_id = txn_data.get(REVOC_REG_DEF_ID)
        seq_no = get_seq_no(txn)
        txn_time = get_txn_time(txn)
        assert author_did
        assert revoc_reg_def_id
        assert seq_no
        assert txn_time
        path = RevocRegEntryHandler.make_state_path_for_revoc_reg_entry_accum(revoc_reg_def_id)

        # TODO: do not duplicate seqNo here
        # doing this now just for backward-compatibility
        txn_data = deepcopy(txn_data)
        txn_data[f.SEQ_NO.nm] = seq_no
        txn_data[TXN_TIME] = txn_time
        value_bytes = encode_state_value(txn_data, seq_no, txn_time)
        return path, value_bytes

    @staticmethod
    def prepare_revoc_reg_entry_for_state(txn, path_only=False):
        author_did = get_from(txn)
        txn_data = get_payload_data(txn)
        revoc_reg_def_id = txn_data.get(REVOC_REG_DEF_ID)
        assert author_did
        assert revoc_reg_def_id
        path = RevocRegEntryHandler.make_state_path_for_revoc_reg_entry(revoc_reg_def_id=revoc_reg_def_id)
        if path_only:
            return path

        seq_no = get_seq_no(txn)
        txn_time = get_txn_time(txn)
        assert seq_no
        assert txn_time
        # TODO: do not duplicate seqNo here
        # doing this now just for backward-compatibility
        txn_data = deepcopy(txn_data)
        txn_data[f.SEQ_NO.nm] = seq_no
        txn_data[TXN_TIME] = txn_time
        value_bytes = encode_state_value(txn_data, seq_no, txn_time)
        return path, value_bytes

    @staticmethod
    def make_state_path_for_revoc_reg_entry_accum(revoc_reg_def_id) -> bytes:
        return "{MARKER}:{REVOC_REG_DEF_ID}" \
            .format(MARKER=MARKER_REVOC_REG_ENTRY_ACCUM,
                    REVOC_REG_DEF_ID=revoc_reg_def_id).encode()

    @staticmethod
    def make_state_path_for_revoc_reg_entry(revoc_reg_def_id) -> bytes:
        return "{MARKER}:{REVOC_REG_DEF_ID}" \
            .format(MARKER=MARKER_REVOC_REG_ENTRY,
                    REVOC_REG_DEF_ID=revoc_reg_def_id).encode()



from abc import ABCMeta

from common.serializers.serialization import domain_state_serializer, config_state_serializer
from indy_common.authorize.auth_constraints import ConstraintCreator, ConstraintsSerializer, OFF_LEDGER_SIGNATURE, \
    CONSTRAINT_ID, ConstraintsEnum, AUTH_CONSTRAINTS
from indy_common.authorize.auth_request_validator import WriteRequestValidator
from indy_common.config_util import getConfig
from indy_common.constants import CONFIG_LEDGER_ID, CONSTRAINT
from indy_common.state.state_constants import MARKER_AUTH_RULE
from indy_node.server.request_handlers.config_req_handlers.auth_rule.static_auth_rule_helper import StaticAuthRuleHelper
from plenum.common.exceptions import InvalidClientRequest
from plenum.server.database_manager import DatabaseManager
from plenum.server.request_handlers.handler_interfaces.write_request_handler import WriteRequestHandler


class AbstractAuthRuleHandler(WriteRequestHandler, metaclass=ABCMeta):

    def __init__(self, database_manager: DatabaseManager,
                 write_req_validator: WriteRequestValidator, txn_type):
        super().__init__(database_manager, txn_type, CONFIG_LEDGER_ID)
        self.write_req_validator = write_req_validator
        self.constraint_serializer = ConstraintsSerializer(domain_state_serializer)

    def _static_validation_for_rule(self, operation, identifier, req_id):
        try:
            ConstraintCreator.create_constraint(operation.get(CONSTRAINT))
        except (ValueError, KeyError) as exp:
            raise InvalidClientRequest(identifier,
                                       req_id,
                                       str(exp))
        StaticAuthRuleHelper.check_auth_key(operation, identifier, req_id, self.write_req_validator.auth_map)

    def _update_auth_constraint(self, auth_key: str, constraint):
        self.state.set(AbstractAuthRuleHandler.make_state_path_for_auth_rule(auth_key),
                       self.constraint_serializer.serialize(constraint))

    def _decode_state_value(self, encoded):
        if encoded:
            return config_state_serializer.deserialize(encoded)
        return None

    @staticmethod
    def make_state_path_for_auth_rule(action_id) -> bytes:
        return "{MARKER}:{ACTION_ID}" \
            .format(MARKER=MARKER_AUTH_RULE,
                    ACTION_ID=action_id).encode()



import importlib
import os

from stp_core.common.log import Logger, getlogger
from stp_core.types import HA
from stp_core.loop.looper import Looper

from indy_common.config_helper import NodeConfigHelper
from indy_node.server.node import Node


def integrate(node_config_helper, node, logger):
    plugin_root = node_config_helper.config.PLUGIN_ROOT
    try:
        plugin_root = importlib.import_module(plugin_root)
    except ImportError:
        raise ImportError('Incorrect plugin root {}. No such package found'.
                          format(plugin_root))
    enabled_plugins = node_config_helper.config.ENABLED_PLUGINS
    for plugin_name in enabled_plugins:
        importlib.import_module(plugin_name)
        main = importlib.import_module('{}.main'.format(plugin_name))
        logger.info('Going to integrate plugin: {}'.format(plugin_name))
        node = main.integrate_plugin_in_node(node)
        logger.info('Integrated plugin: {}'.format(plugin_name))
    return node


def run_node(config, name, node_ip, node_port, client_ip, client_port):
    node_ha = HA(node_ip, node_port)
    client_ha = HA(client_ip, client_port)

    node_config_helper = NodeConfigHelper(name, config)

    logFileName = os.path.join(node_config_helper.log_dir, name + ".log")

    logger = getlogger()
    Logger().apply_config(config)
    Logger().enableFileLogging(logFileName)

    logger.setLevel(config.logLevel)
    logger.debug("You can find logs in {}".format(logFileName))

    vars = [var for var in os.environ.keys() if var.startswith("INDY")]
    logger.debug("Indy related env vars: {}".format(vars))

    with Looper(debug=config.LOOPER_DEBUG) as looper:
        node = Node(name,
                    config_helper=node_config_helper,
                    ha=node_ha, cliha=client_ha)
        node = integrate(node_config_helper, node, logger)
        looper.add(node)
        looper.run()



from importlib import import_module

from plenum.common.config_util import getConfig as PlenumConfig, \
    getConfigOnce as PlenumConfigOnce, extend_with_default_external_config


CONFIG = None


def _getConfig(plenum_config_func,
               general_config_dir=None,
               user_config_dir=None,
               ignore_external_config_update_errors=False):
    config = plenum_config_func(general_config_dir)
    indyConfig = import_module("indy_common.config")
    config.__dict__.update(indyConfig.__dict__)

    try:
        extend_with_default_external_config(config,
                                            general_config_dir=general_config_dir,
                                            user_config_dir=user_config_dir)
    except Exception as ex:
        if not ignore_external_config_update_errors:
            raise ex
    return config


def getConfig(general_config_dir=None, user_config_dir=None):
    global CONFIG
    if not CONFIG:
        CONFIG = _getConfig(PlenumConfig, general_config_dir, user_config_dir)
    return CONFIG


def getConfigOnce(general_config_dir=None, user_config_dir=None,
                  ignore_external_config_update_errors=False):
    return _getConfig(PlenumConfigOnce, general_config_dir, user_config_dir,
                      ignore_external_config_update_errors)



from indy_common.authorize.auth_constraints import AuthConstraint, IDENTITY_OWNER, AuthConstraintOr, AuthConstraintAnd
from indy_common.constants import ENDORSER
from indy_common.test.auth.metadata.helper import validate, PLUGIN_FIELD, Action
from plenum.common.constants import TRUSTEE, STEWARD

MAX_SIG_COUNT = 3


def test_plugin_or_rule_all_amount_trustee_no_endorser(write_auth_req_validator, write_request_validation,
                                                       signatures, is_owner, amount):
    validate(
        auth_constraint=AuthConstraintOr(auth_constraints=[
            AuthConstraint(role=TRUSTEE, sig_count=1, need_to_be_owner=False,
                           metadata={PLUGIN_FIELD: 1}),
            AuthConstraint(role=STEWARD, sig_count=1, need_to_be_owner=False,
                           metadata={PLUGIN_FIELD: 2}),
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=True,
                           metadata={PLUGIN_FIELD: 3}),
        ]),
        valid_actions=[Action(author=TRUSTEE, endorser=None, sigs={TRUSTEE: s}, is_owner=owner, amount=1,
                              extra_sigs=True)
                       for s in range(1, 4)
                       for owner in [True, False]
                       ] + [
            Action(author=TRUSTEE, endorser=None, sigs={TRUSTEE: s1,
                                                        STEWARD: s2},
                   is_owner=owner, amount=2, extra_sigs=True)
            for s1 in range(1, 4)
            for s2 in range(1, 4)
            for owner in [True, False]] + [
            Action(author=TRUSTEE, endorser=None, sigs={TRUSTEE: s1, ENDORSER: s2},
                   is_owner=True, amount=3, extra_sigs=True)
            for s1 in range(1, 4)
            for s2 in range(1, 4)],
        author=TRUSTEE, endorser=None,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )


def test_plugin_or_rule_one_amount_same_role_endorser_no_endorser(write_auth_req_validator, write_request_validation,
                                                                  signatures, is_owner, amount):
    validate(
        auth_constraint=AuthConstraintOr(auth_constraints=[
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=False),
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=False,
                           metadata={PLUGIN_FIELD: 2}),
        ]),
        valid_actions=[Action(author=ENDORSER, endorser=None, sigs={ENDORSER: s},
                              is_owner=owner, amount=amount, extra_sigs=True)
                       for s in range(1, 4)
                       for owner in [True, False]
                       for amount in [2, None]],
        author=ENDORSER, endorser=None,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )


def test_plugin_or_rule_one_amount_same_role_endorser_endorser(write_auth_req_validator, write_request_validation,
                                                               signatures, is_owner, amount):
    validate(
        auth_constraint=AuthConstraintOr(auth_constraints=[
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=False),
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=False,
                           metadata={PLUGIN_FIELD: 2}),
        ]),
        valid_actions=[Action(author=ENDORSER, endorser=ENDORSER, sigs={ENDORSER: s},
                              is_owner=owner, amount=amount, extra_sigs=True)
                       for s in range(1, 4)
                       for owner in [True, False]
                       for amount in [2, None]],
        author=ENDORSER, endorser=ENDORSER,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )


def test_plugin_or_rule_one_amount_same_role_owner_no_endorser(write_auth_req_validator, write_request_validation,
                                                               signatures, is_owner, amount):
    validate(
        auth_constraint=AuthConstraintOr(auth_constraints=[
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=False),
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=False,
                           metadata={PLUGIN_FIELD: 2}),
        ]),
        valid_actions=[],
        author=IDENTITY_OWNER, endorser=None,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )


def test_plugin_or_rule_one_amount_same_role_owner_endorser(write_auth_req_validator, write_request_validation,
                                                            signatures, is_owner, amount):
    validate(
        auth_constraint=AuthConstraintOr(auth_constraints=[
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=False),
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=False,
                           metadata={PLUGIN_FIELD: 2}),
        ]),
        valid_actions=[Action(author=IDENTITY_OWNER, endorser=ENDORSER, sigs={ENDORSER: s1, IDENTITY_OWNER: s2},
                              is_owner=owner, amount=amount, extra_sigs=True)
                       for s1 in range(1, 4)
                       for s2 in range(1, 4)
                       for owner in [True, False]
                       for amount in [2, None]],
        author=IDENTITY_OWNER, endorser=ENDORSER,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )


def test_plugin_or_rule_one_amount_diff_roles_endorser_no_endorser(write_auth_req_validator, write_request_validation,
                                                                   signatures, is_owner, amount):
    validate(
        auth_constraint=AuthConstraintOr(auth_constraints=[
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=False),
            AuthConstraint(role=IDENTITY_OWNER, sig_count=1, need_to_be_owner=True,
                           metadata={PLUGIN_FIELD: 1}),
        ]),
        valid_actions=[Action(author=ENDORSER, endorser=None, sigs={ENDORSER: s},
                              is_owner=owner, amount=None, extra_sigs=True)
                       for s in range(1, 4)
                       for owner in [True, False]] + [
            Action(author=ENDORSER, endorser=None, sigs={ENDORSER: s1,
                                                         IDENTITY_OWNER: s2},
                   is_owner=True, amount=1, extra_sigs=True)
            for s1 in range(1, 4)
            for s2 in range(1, 4)],
        author=ENDORSER, endorser=None,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )


def test_plugin_or_rule_one_amount_diff_roles_owner_no_endorser(write_auth_req_validator, write_request_validation,
                                                                signatures, is_owner, amount):
    validate(
        auth_constraint=AuthConstraintOr(auth_constraints=[
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=False),
            AuthConstraint(role=IDENTITY_OWNER, sig_count=1, need_to_be_owner=True,
                           metadata={PLUGIN_FIELD: 1}),
        ]),
        valid_actions=[Action(author=IDENTITY_OWNER, endorser=None, sigs={IDENTITY_OWNER: s},
                              is_owner=True, amount=1, extra_sigs=False)
                       for s in range(1, MAX_SIG_COUNT + 1)],
        author=IDENTITY_OWNER, endorser=None,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )


def test_plugin_or_rule_one_amount_diff_roles_owner_endorser(write_auth_req_validator, write_request_validation,
                                                             signatures, is_owner, amount):
    validate(
        auth_constraint=AuthConstraintOr(auth_constraints=[
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=False),
            AuthConstraint(role=IDENTITY_OWNER, sig_count=1, need_to_be_owner=True,
                           metadata={PLUGIN_FIELD: 1}),
        ]),
        valid_actions=[Action(author=IDENTITY_OWNER, endorser=ENDORSER, sigs={IDENTITY_OWNER: s1, ENDORSER: s2},
                              is_owner=owner, amount=None, extra_sigs=True)
                       for s1 in range(1, 4)
                       for s2 in range(1, 4)
                       for owner in [True, False]] + [
            Action(author=IDENTITY_OWNER, endorser=ENDORSER, sigs={IDENTITY_OWNER: s1,
                                                                   ENDORSER: s2},
                   is_owner=True, amount=1, extra_sigs=True)
            for s1 in range(1, 4)
            for s2 in range(1, 4)],
        author=IDENTITY_OWNER, endorser=ENDORSER,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )


def test_plugin_or_rule_one_amount_all_roles_endorser_no_endorser(write_auth_req_validator, write_request_validation,
                                                                  signatures, is_owner, amount, off_ledger_signature):
    validate(
        auth_constraint=AuthConstraintOr(auth_constraints=[
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=False),
            AuthConstraint(role='*', sig_count=1, need_to_be_owner=True,
                           off_ledger_signature=off_ledger_signature,
                           metadata={PLUGIN_FIELD: 3}),
        ]),
        valid_actions=[Action(author=ENDORSER, endorser=None, sigs={ENDORSER: s},
                              is_owner=owner, amount=None, extra_sigs=True)
                       for s in range(1, 4)
                       for owner in [True, False]] + [
            Action(author=ENDORSER, endorser=None, sigs={ENDORSER: s}, is_owner=True, amount=3, extra_sigs=True)
            for s in range(1, 4)],
        author=ENDORSER, endorser=None,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )


def test_plugin_or_rule_one_amount_all_roles_owner_no_endorser(write_auth_req_validator, write_request_validation,
                                                               signatures, is_owner, amount, off_ledger_signature):
    validate(
        auth_constraint=AuthConstraintOr(auth_constraints=[
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=False),
            AuthConstraint(role='*', sig_count=1, need_to_be_owner=True,
                           off_ledger_signature=off_ledger_signature,
                           metadata={PLUGIN_FIELD: 3}),
        ]),
        valid_actions=[Action(author=IDENTITY_OWNER, endorser=None, sigs={IDENTITY_OWNER: s},
                              is_owner=True, amount=3, extra_sigs=False)
                       for s in range(1, MAX_SIG_COUNT + 1)],
        author=IDENTITY_OWNER, endorser=None,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )


def test_plugin_or_rule_one_amount_all_roles_owner_endorser(write_auth_req_validator, write_request_validation,
                                                            signatures, is_owner, amount, off_ledger_signature):
    validate(
        auth_constraint=AuthConstraintOr(auth_constraints=[
            AuthConstraint(role=ENDORSER, sig_count=1, need_to_be_owner=False),
            AuthConstraint(role='*', sig_count=1, need_to_be_owner=True,
                           off_ledger_signature=off_ledger_signature,
                           metadata={PLUGIN_FIELD: 3}),
        ]),
        valid_actions=[Action(author=IDENTITY_OWNER, endorser=ENDORSER, sigs={IDENTITY_OWNER: s1, ENDORSER: s2},
                              is_owner=owner, amount=None, extra_sigs=True)
                       for s1 in range(1, 4)
                       for s2 in range(1, 4)
                       for owner in [True, False]] + [
            Action(author=IDENTITY_OWNER, endorser=ENDORSER, sigs={IDENTITY_OWNER: s1, ENDORSER: s2},
                   is_owner=True, amount=3, extra_sigs=True)
            for s1 in range(1, 4)
            for s2 in range(1, 4)],
        author=IDENTITY_OWNER, endorser=ENDORSER,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )


def test_plugin_or_rule_diff_amount_same_role_endorser_no_endorser(write_auth_req_validator, write_request_validation,
                                                                   signatures, is_owner, amount):
    validate(
        auth_constraint=AuthConstraintOr(auth_constraints=[
            AuthConstraint(role=ENDORSER, sig_count=2, need_to_be_owner=False,
                           metadata={PLUGIN_FIELD: 2}),
            AuthConstraint(role=ENDORSER, sig_count=3, need_to_be_owner=False,
                           metadata={PLUGIN_FIELD: 1}),
        ]),
        valid_actions=[
            Action(author=ENDORSER, endorser=None, sigs={ENDORSER: 2},
                   is_owner=True, amount=2, extra_sigs=True),
            Action(author=ENDORSER, endorser=None, sigs={ENDORSER: 3},
                   is_owner=True, amount=1, extra_sigs=True),
            Action(author=ENDORSER, endorser=None, sigs={ENDORSER: 2},
                   is_owner=False, amount=2, extra_sigs=True),
            Action(author=ENDORSER, endorser=None, sigs={ENDORSER: 3},
                   is_owner=False, amount=1, extra_sigs=True)
        ],
        author=ENDORSER, endorser=None,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )


def test_plugin_or_rule_diff_amount_same_role_owner_no_endorser(write_auth_req_validator, write_request_validation,
                                                                signatures, is_owner, amount):
    validate(
        auth_constraint=AuthConstraintOr(auth_constraints=[
            AuthConstraint(role=ENDORSER, sig_count=2, need_to_be_owner=False,
                           metadata={PLUGIN_FIELD: 2}),
            AuthConstraint(role=ENDORSER, sig_count=3, need_to_be_owner=False,
                           metadata={PLUGIN_FIELD: 1}),
        ]),
        valid_actions=[],
        author=IDENTITY_OWNER, endorser=None,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )


def test_plugin_or_rule_diff_amount_same_role_owner_endorser(write_auth_req_validator, write_request_validation,
                                                             signatures, is_owner, amount):
    validate(
        auth_constraint=AuthConstraintOr(auth_constraints=[
            AuthConstraint(role=ENDORSER, sig_count=2, need_to_be_owner=False,
                           metadata={PLUGIN_FIELD: 2}),
            AuthConstraint(role=ENDORSER, sig_count=3, need_to_be_owner=False,
                           metadata={PLUGIN_FIELD: 1}),
        ]),
        valid_actions=[Action(author=IDENTITY_OWNER, endorser=ENDORSER, sigs={IDENTITY_OWNER: s, ENDORSER: 2},
                              is_owner=owner, amount=2, extra_sigs=True)
                       for s in range(1, 4)
                       for owner in [True, False]] + [
            Action(author=IDENTITY_OWNER, endorser=ENDORSER, sigs={IDENTITY_OWNER: s,
                                                                   ENDORSER: 3},
                   is_owner=owner, amount=1, extra_sigs=True)
            for s in range(1, 4)
            for owner in [True, False]],
        author=IDENTITY_OWNER, endorser=ENDORSER,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )


def test_plugin_and_rule_trustee_no_endorser(write_auth_req_validator, write_request_validation,
                                             signatures, is_owner, amount):
    validate(
        auth_constraint=AuthConstraintAnd(auth_constraints=[
            AuthConstraint(role=TRUSTEE, sig_count=2, need_to_be_owner=False,
                           metadata={PLUGIN_FIELD: 2}),
            AuthConstraint(role=STEWARD, sig_count=3, need_to_be_owner=False,
                           metadata={PLUGIN_FIELD: 2})
        ]),
        valid_actions=[
            Action(author=TRUSTEE, endorser=None, sigs={TRUSTEE: s, STEWARD: 3},
                   is_owner=owner, amount=2, extra_sigs=True)
            for s in range(2, 4)
            for owner in [True, False]
        ],
        author=TRUSTEE, endorser=None,
        all_signatures=signatures, is_owner=is_owner, amount=amount,
        write_auth_req_validator=write_auth_req_validator,
        write_request_validation=write_request_validation
    )



import json

import pytest
from indy_common.constants import ATTRIB
from indy_node.persistence.attribute_store import AttributeStore

from indy_node.server.request_handlers.domain_req_handlers.attribute_handler import AttributeHandler
from indy_node.test.request_handlers.helper import add_to_idr
from plenum.common.constants import ENC, ATTRIB_LABEL
from plenum.common.exceptions import InvalidClientRequest, UnauthorizedClientRequest
from plenum.common.request import Request
from plenum.common.txn_util import reqToTxn, append_txn_metadata
from plenum.common.util import randomString

from indy_node.test.attrib_txn.test_nym_attrib import attributeData, attributeName, attributeValue
from plenum.test.helper import sdk_multisign_request_object, sdk_sign_request_objects
from plenum.test.testing_utils import FakeSomething
from storage.kv_in_memory import KeyValueStorageInMemory


@pytest.fixture(scope="module")
def attrib_handler(db_manager, write_auth_req_validator):
    return AttributeHandler(db_manager, write_auth_req_validator)


@pytest.fixture(scope="function")
def attrib_request(attributeData, looper, sdk_wallet_client):
    return Request(identifier=sdk_wallet_client[1],
                   reqId=5,
                   signature="signature",
                   operation={'type': ATTRIB,
                              'dest': randomString(),
                              'raw': attributeData})


def test_attrib_static_validation_passes(attrib_request, attrib_handler: AttributeHandler):
    attrib_handler.static_validation(attrib_request)


def test_attrib_static_validation_fails(attrib_request, attrib_handler: AttributeHandler):
    attrib_request.operation[ENC] = randomString()
    with pytest.raises(InvalidClientRequest):
        attrib_handler.static_validation(attrib_request)


def test_attrib_dynamic_validation_fails(attrib_request, attrib_handler: AttributeHandler):
    with pytest.raises(InvalidClientRequest):
        attrib_handler.dynamic_validation(attrib_request, 0)


def test_attrib_dynamic_validation_fails_not_owner(attrib_request, attrib_handler: AttributeHandler):
    add_to_idr(attrib_handler.database_manager.idr_cache, attrib_request.operation['dest'], None)
    with pytest.raises(UnauthorizedClientRequest):
        attrib_handler.dynamic_validation(attrib_request, 0)


def test_attrib_dynamic_validation_passes(attrib_request, attrib_handler: AttributeHandler):
    add_to_idr(attrib_handler.database_manager.idr_cache, attrib_request.operation['dest'], None)
    attrib_request._identifier = attrib_request.operation['dest']
    attrib_handler.dynamic_validation(attrib_request, 0)


def test_update_state(attrib_handler, attrib_request):
    seq_no = 1
    txn_time = 1560241033
    txn_id = "id"
    txn = reqToTxn(attrib_request)
    append_txn_metadata(txn, seq_no, txn_time, txn_id)
    attr_type, path, value, hashed_value, value_bytes = AttributeHandler.prepare_attr_for_state(txn)

    attrib_handler.update_state(txn, None, attrib_request)
    assert attrib_handler.get_from_state(path) == (hashed_value, seq_no, txn_time)




from plenum.common.exceptions import RequestRejectedException, RequestNackedException
from plenum.test.helper import sdk_get_bad_response
from plenum.test.pool_transactions.helper import sdk_add_new_nym
from indy_node.test.pool_config.helper import sdk_pool_config_sent
from plenum.common.constants import STEWARD_STRING


def test_only_trustee_send_pool_config_writes_true_force_false(
        nodeSet, looper, sdk_pool_handle, sdk_wallet_trustee, poolConfigWTFF):
    sdk_wallet_steward = sdk_add_new_nym(looper, sdk_pool_handle,
                                         sdk_wallet_trustee, 'tmpname', STEWARD_STRING)
    req = sdk_pool_config_sent(looper, sdk_pool_handle, sdk_wallet_steward, poolConfigWTFF)
    sdk_get_bad_response(looper, [req], RequestRejectedException, "Not enough TRUSTEE signatures")


def test_only_trustee_send_pool_config_writes_false_force_false(
        nodeSet, looper, sdk_pool_handle, sdk_wallet_trustee, poolConfigWFFF):
    sdk_wallet_steward = sdk_add_new_nym(looper, sdk_pool_handle,
                                         sdk_wallet_trustee, 'tmpname', STEWARD_STRING)
    req = sdk_pool_config_sent(looper, sdk_pool_handle, sdk_wallet_steward, poolConfigWFFF)
    sdk_get_bad_response(looper, [req], RequestRejectedException, "Not enough TRUSTEE signatures")


def test_only_trustee_send_pool_config_writes_true_force_true(
        nodeSet, looper, sdk_pool_handle, sdk_wallet_trustee, poolConfigWTFT):
    sdk_wallet_steward = sdk_add_new_nym(looper, sdk_pool_handle,
                                         sdk_wallet_trustee, 'tmpname', STEWARD_STRING)
    req = sdk_pool_config_sent(looper, sdk_pool_handle, sdk_wallet_steward, poolConfigWTFT)
    sdk_get_bad_response(looper, [req], RequestNackedException, "Not enough TRUSTEE signatures")


def test_only_trustee_send_pool_config_writes_false_force_true(
        nodeSet, looper, sdk_pool_handle, sdk_wallet_trustee, poolConfigWFFT):
    sdk_wallet_steward = sdk_add_new_nym(looper, sdk_pool_handle,
                                         sdk_wallet_trustee, 'tmpname', STEWARD_STRING)
    req = sdk_pool_config_sent(looper, sdk_pool_handle, sdk_wallet_steward, poolConfigWFFT)
    sdk_get_bad_response(looper, [req], RequestNackedException, "Not enough TRUSTEE signatures")



import pytest

from indy_common.authorize.auth_actions import AuthActionEdit, EDIT_PREFIX
from indy_common.authorize.auth_constraints import AuthConstraint
from indy_common.constants import AUTH_RULE
from indy_node.test.auth_rule.auth_framework.basic import AuthTest
from plenum.common.exceptions import RequestRejectedException
from plenum.test.pool_transactions.helper import sdk_add_new_nym
from indy_common.authorize import auth_map

from indy_node.test.helper import build_auth_rule_request_json
from indy_node.test.auth_rule.helper import sdk_send_and_check_auth_rule_request


class AuthRuleTest(AuthTest):
    def __init__(self, env, action_id):
        super().__init__(env, action_id)

    def prepare(self):
        self.new_default_wallet = sdk_add_new_nym(self.looper, self.sdk_pool_handle, self.trustee_wallet, role=None)
        self.default_auth_rule = self.get_default_auth_rule()
        self.changed_auth_rule = self.get_changed_auth_rule()

    def run(self):
        # Step 1. Check default auth rule
        sdk_send_and_check_auth_rule_request(self.looper, self.sdk_pool_handle, self.trustee_wallet)
        with pytest.raises(RequestRejectedException):
            sdk_send_and_check_auth_rule_request(self.looper, self.sdk_pool_handle, self.new_default_wallet)

        # Step 2. Change auth rule
        self.send_and_check(self.changed_auth_rule, self.trustee_wallet)

        # Step 3. Check, that we cannot send txn the old way
        with pytest.raises(RequestRejectedException):
            sdk_send_and_check_auth_rule_request(self.looper, self.sdk_pool_handle, self.trustee_wallet)

        # Step 4. Check, that new auth rule is used
        sdk_send_and_check_auth_rule_request(self.looper, self.sdk_pool_handle, self.new_default_wallet)

        # Step 5. Return default auth rule
        self.send_and_check(self.default_auth_rule, self.new_default_wallet)

        # Step 6. Check, that default auth rule works
        sdk_send_and_check_auth_rule_request(self.looper, self.sdk_pool_handle, self.trustee_wallet)
        with pytest.raises(RequestRejectedException):
            sdk_send_and_check_auth_rule_request(self.looper, self.sdk_pool_handle, self.new_default_wallet)

    def result(self):
        pass

    def get_default_auth_rule(self):
        action = AuthActionEdit(txn_type=AUTH_RULE,
                                field='*',
                                old_value='*',
                                new_value='*')
        constraint = auth_map.auth_map.get(action.get_action_id())
        return build_auth_rule_request_json(
            self.looper, self.new_default_wallet[1],
            auth_action=EDIT_PREFIX,
            auth_type=AUTH_RULE,
            field='*',
            old_value='*',
            new_value='*',
            constraint=constraint.as_dict
        )

    def get_changed_auth_rule(self):
        constraint = AuthConstraint(role=None,
                                    sig_count=1,
                                    need_to_be_owner=False)
        return build_auth_rule_request_json(
            self.looper, self.trustee_wallet[1],
            auth_action=EDIT_PREFIX,
            auth_type=AUTH_RULE,
            field='*',
            old_value='*',
            new_value='*',
            constraint=constraint.as_dict
        )
